/*
 * @(#)AHA.java	1.0	98/08/03
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.io.*;
import java.lang.String;
import java.util.Hashtable;
import javax.servlet.*;
import javax.servlet.http.*;
/**
 *
 * A class which is used to store and retrieve references to objects
 * used throughout the processing of a request.
 * Such objects are used to provide parts of the functionality of AHA,
 * and they work closely together.
 * Examples of objects to which AHA contains references are a "Messages"
 * object (for generating error messages), an "Output" object (which is
 * a PrintWriter) used for everything that is sent to the browser,
 * A "UserModel" object which contains the user model, etc.
 *
 */
public class AHA {

	private ServletContext ServletContext;

	public ServletContext getServletContext() { return ServletContext;}
	public void setServletContext(ServletContext s) {
		ServletContext = s;
	}
	private String ReqListFile = "xmlreqlist";
	private String GenListFile = "xmlgenlist";
	private String UserChangeableConceptsFile = "changeableconcepts";
	private String DocumentsList = "doclist";

  // All data fields of AHA are private and accessible only through
  // get and set methods.
  
  private String Host = null;
  // The name or IP number of the server.
  // This is needed to tell the Stop applet where the server is.

  private int Port = -1;
  // The port number of the server. Also needed by Stop.

  private String HtDocs = null;
  // The DocumentRoot of the WWW server. In theory it should
  // be possible to deduce this from some environment info
  // (like PATH_INFO and PATH_TRANSLATED) but alas this
  // doesn't appear to work consistently accross platforms
  // and servers, so we just initialize it at startup.

  private String Course = null;
  // The name of the directory of the AHA document (relative
  // to the DocumentRoot).

  private String Logs = null;
  // Where the server keeps log files. AHA will put the
  // logs in a subdirectory "Course" of this dir.

  private String Title = null;
  // The title of the AHA document. Is also used as the title
  // of each HTML page.

  private String Background = null;
  // The name of the background image (in the "icons" dir).

  // Setting of the language for messages.
  public static final int English = 0;
  public static final int Dutch = 1;
  public static final String Languages[] = { "English", "Dutch" };
  private int Language = English;

  // The following strings can be changed to whatever you want,
  // but we suggest just leaving them. It doesn't hurt, and why would
  // you even care what these files or directories are called.
  private String Docs = "html";
  private String Test = "test";
  private String Icons = "icons";
  private String Applets = "applets";
  private String Header = "header";
  private String Footer = "footer";
  private String Knowlist = "knowlist";
  private String FirstDoc = "index.xml";

	private String servlets = "/servlet/AHA";

	public String getServletDir() {
		return servlets;
	}
  private Messages Messages = null;
  // The Messages object used for error messages, headers and footers.

  private UserModel UserModel = null;
  // The object containint the user model.

  private PrintWriter Writer = null;
  // The output channel.

	private GenerateList GenerateList = null;
	private RequirementList RequirementList = null;


	public AHA() {
	}

  /**
   * The only constructor registers the pre-initialized attributes.
   */
  public AHA(String HtDocs, String Logs, String Course,
		  String Title, String Background, String Language)
  {
    this.HtDocs = HtDocs;
    this.Logs = Logs;
    this.Course = Course;
    this.Title = Title;
    this.Background = Background;
    setLanguage(Language);
  }

  /**
   * Return the HtDocs field.
   */
  public String getHtDocs() {
    return HtDocs;
  }

  /**
   * Set the HtDocs field.
   */
  public void setHtDocs(String HtDocs) {
    this.HtDocs = HtDocs;
  }

  /**
   * Return the Logs field.
   */
  public String getLogs() {
    return Logs;
  }

  /**
   * Set the Logs field.
   */
  public void setLogs(String Logs) {
    this.Logs = Logs;
  }

  /**
   * Return the Course field.
   */
  public String getCourse() {
    return Course;
  }

  /**
   * Set the Course field.
   */
  public void setCourse(String Course) {
    this.Course = Course;
  }

  /**
   * Return the Title field.
   */
  public String getTitle() {
    return Title;
  }

  /**
   * Set the Title field.
   */
  public void setTitle(String Title) {
    this.Title = Title;
  }

  /**
   * Return the Background field.
   */
  public String getBackground() {
    return Background;
  }

  /**
   * Set the Background field.
   */
  public void setBackground(String Background) {
    this.Background = Background;
  }


	public String getLanguageName(int i) {
		if (0 <= i && i <=Languages.length) {
			return Languages[i];
		} else 
			return "English";
		
	}

  /**
   * Return the Language field.
   */
  public int getLanguage() {
    return Language;
  }

  /**
   * Set the Language field.
   * Currently only Dutch and English are supported.
   * The default language is English.
   * The language switch is only available for regular messages,
   * thus not for error messages. (Sorry, maybe in a future version...)
   */
  public void setLanguage(String Language) {
    int i;
    this.Language = 0; // default: English
    for (i = 0; i < Languages.length; ++i)
      if (Languages[i].equalsIgnoreCase(Language))
	this.Language = i;
  }
  
  /**
   * Return the Docs field.
   */
  public String getDocs() {
    return Docs;
  }

  /**
   * Set the Docs field.
   */
  public void setDocs(String Docs) {
    this.Docs = Docs;
  }

  /**
   * Return the Test field.
   */
  public String getTest() {
    return Test;
  }

  /**
   * Set the Test field.
   */
  public void setTest(String Test) {
    this.Test = Test;
  }

  /**
   * Return the Icons field.
   */
  public String getIcons() {
    return Icons;
  }

  /**
   * Set the Icons field.
   */
  public void setIcons(String Icons) {
    this.Icons = Icons;
  }

  /**
   * Return the Applets field.
   */
  public String getApplets() {
    return Applets;
  }

  /**
   * Set the Applets field.
   */
  public void setApplets(String Applets) {
    this.Applets = Applets;
  }

  /**
   * Return the Header field.
   */
  public String getHeader() {
    return Header;
  }

  /**
   * Set the Header field.
   */
  public void setHeader(String Header) {
    this.Header = Header;
  }

  /**
   * Return the Footer field.
   */
  public String getFooter() {
    return Footer;
  }

  /**
   * Set the Footer field.
   */
  public void setFooter(String Footer) {
    this.Footer = Footer;
  }

  /**
   * Return the Knowlist field.
   */
  public String getKnowlist() {
    return Knowlist;
  }

  /**
   * Set the Knowlist field.
   */
  public void setKnowlist(String Knowlist) {
    this.Knowlist = Knowlist;
  }

  /**
   * Return the FirstDoc field.
   */
  public String getFirstDoc() {
    return FirstDoc;
  }

  /**
   * Set the FirstDoc field.
   */
  public void setFirstDoc(String FirstDoc) {
    this.FirstDoc = FirstDoc;
  }


  /**
   * Return the Messages field.
   */
  public Messages getMessages() {
    return Messages;
  }

  /**
   * Set the Messages field.
   */
  public void setMessages(Messages Messages) {
    this.Messages = Messages;
  }

  /**
   * Return the Host field.
   */
  public String getHost() {
    return Host;
  }

  /**
   * Set the Host field.
   */
  public void setHost(String Host) {
    this.Host = Host;
  }

  /**
   * Return the Port field.
   */
  public int getPort() {
    return Port;
  }

  /**
   * Set the Port field.
   */
  public void setPort(int Port) {
    this.Port = Port;
  }

  /**
   * Return the UserModel field.
   */
  public UserModel getUserModel() {
    return UserModel;
  }

  /**
   * Set the UserModel field.
   */
  public void setUserModel(UserModel UserModel) {
    this.UserModel = UserModel;
  }

  /**
   * Return the Writer field.
   */
  public PrintWriter getWriter() {
    return Writer;
  }

  /**
   * Set the Writer field.
   */
  public void setWriter(PrintWriter Writer) {
    this.Writer = Writer;
  }

	public GenerateList getGenerateList() {
		return GenerateList;
	}
	public void setGenerateList(GenerateList hs) {
		GenerateList = hs;
	}

	public RequirementList getRequirementList() {
		return RequirementList;
	}
	public void setRequirementList(RequirementList hs) {
		RequirementList = hs;
	}

	public String getLoginWindow() {
		if (Course == null) return "/";
		return "/"+Course;
	}
	public String getUserChangeableConceptsFile() {
		return HtDocs+"/"+Course+"/"+UserChangeableConceptsFile;
	}
	public String getReqListFile() {
		return HtDocs+"/"+Course+"/"+ReqListFile;
	}
	public String getGenListFile() {
		return HtDocs+"/"+Course+"/"+GenListFile;
	}
	public String getDocumentsList() {
		return HtDocs+"/"+Course+"/"+DocumentsList;
	}

	public String getBase() {
		//return getServletDir();
		return "";
	}	
}
