/**
 * @(#)Entree.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.io.*;
import javax.servlet.http.*;
import javax.servlet.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Enumeration;

/**
 * The class Entree contains several actions which have to be
 * performed when the user logs in. 
 */

public class Entree extends AhaServlet {

	protected String passwd;
	private boolean debug = false;
	public void println(Object s) { if (debug) System.out.println(s.toString());}
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
	}
		
	/** 
	 * This method checks if a file with name dirname exists.
	 */
	public boolean exists(String dirname) {
		boolean b=false;
		File dir = new File(dirname);
		b = dir.exists();
		return b;
	}	

	/**
	 * The method hasGoodPasswd checks whether the user has
	 * filled in the correct password.
	 */
	public boolean hasGoodPasswd(String loginpasswd, String passwd) {
		return passwd.equals(loginpasswd);
	}

	/**
	 * This method is used only for reading the parameters
	 * passed by the form, and it also initializes the printwriter out
	 */
	protected void service(HttpServletRequest request, HttpServletResponse response) {
	}

	protected void prepare(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		setPrintWriter(request, response);
		readFormParameters(request, response);
	}

	protected void readFormParameters(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		// The input parameters from the login/registration form 
		// necessary to build an AHA object and identification
		try {
			loginid = request.getParameter("id").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No id parameter in form.");
		}
		try {
			passwd = request.getParameter("passwd").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No passwd parameter in form.");
		}
		String www;
		try {
			www = request.getParameter("www").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No www parameter in form.");
		}

		// The userdir string must contain the directory where
		// the server was started. 
		String userdir = System.getProperty("user.dir");

		// HtDocs must be userdir/www 
		HtDocs = new FileName(userdir, www).toString();


		println("HtDocs = "+HtDocs);
		try {
			Course = request.getParameter("course").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No course parameter in form.");
		}
		String logsrelative;
		try {
			logsrelative = request.getParameter("logs").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No logs parameter in form.");
		}

		Logs = new FileName(userdir, logsrelative).toString();

		println("Logs = "+Logs);
		try {	
			Title = request.getParameter("title").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No title parameter in form.");
		}

		try {
			Background = request.getParameter("background").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No background parameter in form.");
		}

		try {
			Language = request.getParameter("language").trim();
		} catch (NullPointerException npe) {
			throw new AhaException("No language parameter in form.");
		}
	}

	protected void setPrintWriter(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		try {
			out = response.getWriter();
		} catch (IOException ioe) {
			System.out.println("No writer.");
			throw new AhaException(this.getName()+ " Error getting outputwriter.");
		}
	}

	/**
	 * The method buildAHA builds a new AHA object with given parameters
	 */
	public void buildAHA(HttpServletRequest request, HttpServletResponse response) throws AhaException{
      	        String uri = javax.servlet.http.HttpUtils.getRequestURL(request).toString();
		String referer = request.getHeader("referer");
		System.out.println(uri);
		System.out.println(referer);

		// all variables are known, now build the AHA object
		AHA = new AHA(HtDocs, Logs, Course, Title, Background, Language);
		String usermodeldir = new FileName(Logs, Course, loginid).toString();
		boolean exist = exists(new FileName(usermodeldir,"model").toString());
		if (exist) {
			try {
				UserModel = new UserModel(AHA,usermodeldir);
			} catch (Exception e) {
				throw new AhaException("The usermodel of "+loginid+" is not readable.");
			}
			AHA.setUserModel(UserModel);
			if (!hasGoodPasswd(passwd, UserModel.getPasswd())) {
				System.out.println("User " + loginid + " filled in a wrong password.");
				throw new AhaException("User " + loginid + " filled in a wrong password.");
			}
		} else {
			throw new AhaException("User " + loginid + "does not exist.");
		}
	}

	public void addAHAInformation(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		// Add some information to AHA
		try {
	      	        String uri = javax.servlet.http.HttpUtils.getRequestURL(request).toString();
			System.out.println(uri);
        	        URL url = new URL(uri);
			Host = url.getHost();
	                Port = url.getPort();
		        if (Port <0 ) {
        			// this should not happen
				Port = 80;
			}

			AHA.setHost(Host);
			AHA.setPort(Port);
		} catch (MalformedURLException mue) {
			System.out.println("Host en Port zijn niet goed");
			throw new AhaException(mue.toString());
		}
		try {
			AhaProperties AhaProperties = new AhaProperties(AHA);
			AHA.setAhaProperties(AhaProperties);
		} catch (AhaException ahae) {
			throw new AhaException(ahae.toString());
		}

		GenerateList GenerateList = new GenerateList(AHA);
		AHA.setGenerateList(GenerateList);
		RequirementList RequirementList = new RequirementList(AHA);
		AHA.setRequirementList(RequirementList);
		Messages Messages = new Messages(AHA);
		AHA.setMessages(Messages);
	}

	public void addSessionInformation(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		// Now make a session and put AHA in it;
		HttpSession session;
		session = request.getSession(false);
		if (session == null) {
			session = request.getSession(true);
		} else  {
			System.out.println("Er is al een sessie.");
		}
		session.setMaxInactiveInterval(-1);
		AHASession ses = new AHASession(AHA, session);
		ServletContext sc = (ServletContext)this.getServletContext();
		AHASession ses2 = (AHASession)sc.getAttribute(loginid+"@"+Course);
		if (ses2 != null) {
			System.out.println("Dezelfde user zit al op dezelfde cursus ingelogd.");
			System.out.println("Het systeem overschrijft de oude sessie.");
		}
		sc.setAttribute(loginid+"@"+Course, ses);
		
		System.out.println("User "+loginid+" logged in.");
	}
	
	public void redirect(HttpServletRequest request, HttpServletResponse response) {
		String redirecturi = response.encodeRedirectURL(AHA.getServletDir() +"/Get/" + Course +"/"+ loginid +"/"+ AHA.getFirstDocument());
		println(redirecturi);
		try {
			response.sendRedirect(redirecturi);
		} catch (IOException ioe) {System.out.println(ioe.toString());}
	}

	// Not used: CHANGE
	public void CheckOtherSessions(ServletContext sc) {
		Enumeration e = sc.getAttributeNames();
		while (e.hasMoreElements()) {
			String key = (String)e.nextElement();
			AHASession ses = (AHASession)sc.getAttribute(key);
			System.out.println(key+", "+ses.getSession().getId());
		}
	}
}