/*
 * @(#)EvaluateTest.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.net.*;
import java.util.*;
import java.text.*;
import java.awt.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The class EvaluateTest evaluates the multiple choice test answers
 * which the user filled in in a test.
 * Some knowledge can be generated if enough correct answers are given.
 */

public class EvaluateTest extends AhaServlet {

	private boolean debug = false;

	public void println(Object s) {if (debug) System.out.println(s.toString());}
	public void print(Object s) {if (debug) System.out.print(s.toString());}
	
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
	}

	protected void service(	HttpServletRequest request, 
				HttpServletResponse response) {
		try {
			super.initService(request,response);
		} catch (AhaException ahae) {
			System.out.println(getName()+" error: "+ahae.toString());
			super.redirect(request, response);
			return;
		}

		String Score;
		try {
			// get a CgiUtil object from the inputstream generated by a form
			CgiUtil Util = new CgiUtil(request.getInputStream(), request.getContentLength());
                        String uri = request.getRequestURI();
			String test;

			// derive the name of the test that was filled in, from the url
                        if (uri.indexOf("/EvaluateTest/")>-1) {
				test = new Derive(uri).getTest();
                        } else {
                                throw new AhaException("Error: wrong test call. Please notify the author about this error.");
                        }
			// build an absolute path to the test file
			String FullDocName = new FileName(HtDocs, Course, AHA.getTest(), test).toString();

			// check if the testfile is readable, otherwise throw an exception
			File docfile = new File(FullDocName);
			if (!docfile.canRead())
				throw new AhaException("Cannot access the test file "+FullDocName+" on the server. Please notify the author about this.");

			// build a new filtertest
			FilterTest filtertest = new FilterTest(AHA, FullDocName);
			// evaluate the answers that the user filled in in the test
			Score = filtertest.EvalQuestions(Util);
			println(Score);
		} catch (Exception e) {
			out.println("EvaluateTest Error: "+e.toString());
		}
	}
}

