/*
 * @(#)FilterTest.java	1.0	98/08/03
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;
import XmlUtils.*;
import java.lang.String;
import java.io.*;
import java.util.Vector;
import java.util.Random;
import java.util.Enumeration;
import org.w3c.dom.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


/**
 * FilterTest is an extension of Filter, to generate multiple-choice tests.
 */
public class FilterTest { // extends Filter {

	private AHA AHA;
	protected UserModel UserModel;
	protected PrintWriter out;

	private String Opening="{{";  
	private String Closing="}}";

	private int TotalQuestions; // Total number of questions in file	  
	private int QuestionsToAsk; // Number of questions to ask 
	private String Title=null;  // Title of the page
	private String Knowledge=null; // Knowledge gained
	private Vector Questions=new Vector(5,5); // vector in which questions + extra data is saved
	private boolean Verbose; 
	private Random R; 
	private Vector alist; /* vector of integer arrays with indices of answers */
	/**
	 * The default constructor is not valid.
	 *
	 * @exception	AhaException	Always thrown because this constructor is
	 * not valid.
	 */
	FilterTest() throws AhaException {
		throw new AhaException("Cannot filter a test without filename and user model.");
	}

	/**
	 * This constructor filters a test according to a user model and page
	 * list. It writes the output to the PrintWriter.
	 *
	 * @exception	AhaException	If an error occurs in the file with the test.
	 */
	public FilterTest(AHA AHA, String FullDocName) throws AhaException{
		this.AHA = AHA;
		this.UserModel = AHA.getUserModel();
		this.out = AHA.getWriter();

		alist=new Vector(5,5);	
		int i,j;	
  	
		//$ Parse the XML file
		//XmlDocument doc=parseFile(FullDocName);
		XmlReader reader;
		try {
			reader = new XmlReader(FullDocName);
		} catch (Exception e) {
			throw new AhaException("Cannot read the file "+FullDocName+". Please notify the author.");
		}
		Element el = reader.getRoot();
		//    //$ assign the root element of the file to el
		//    Element el = doc.getDocumentElement();

		//$ fill the question array with questions, answers etc.

		fillArrays(el);	

		Question q;
		// make sure we have the correct text for the embedded links.
		for (i = 0; i < Questions.size(); ++i) {

			// filter possible links, to ensure the correct link colors.
			q = (Question)Questions.elementAt(i);
			q.setQuestion(FilterLinks(q.getQuestion()));
			Vector V = q.getBadAnswers();
			for (j=0; j < V.size(); ++j)
				V.setElementAt(FilterLinks((String)V.elementAt(j)), j);
			V = q.getGoodAnswers();
			for (j=0; j < V.size(); ++j)
				V.setElementAt(FilterLinks((String)V.elementAt(j)), j);
			V = q.getBadFeedback();
			for (j=0; j < V.size(); ++j)
				V.setElementAt(FilterLinks((String)V.elementAt(j)), j);
			V = q.getGoodFeedback();
			for (j=0; j < V.size(); ++j)
				V.setElementAt(FilterLinks((String)V.elementAt(j)), j);
		}		
	}

	public String HtmlConvert(String tempstr) {
		/* pre:  there is a string which might contain {{ }}
		post: the string no longer contains any of the Opening or Closing markers. 
		They are converted to < and > */

		//$ the positions of the opening and closing markers		
		int openpos,closepos;
 
		//$ convert String to StringBuffer so we can use replace
		StringBuffer tempbuf=new StringBuffer(tempstr);
		while (((openpos=tempstr.indexOf(Opening)) != -1) && ((tempstr.indexOf(Closing)) != -1)) { 

			//$ remove Opening marker and insert "<"
			tempbuf=tempbuf.replace(openpos,openpos+2,"<");
			tempstr=tempbuf.toString();
			//$ closepos will now be assigned to the position of the Closing markers
			closepos=tempstr.indexOf(Closing);
			//$ remove Closing marker and insert ">"
			tempbuf=tempbuf.replace(closepos,closepos+2,">");
			tempstr=tempbuf.toString();
		}				
		//$ return the converted string
		return tempbuf.toString();
	}


	public void getReason(org.w3c.dom.NodeList nodelist, int question,boolean correct) {
		/* pre:  nodelist contains the answer of the questions, question the number of the
		 active question, and correct is true if the answer is a correct one.
		ost: the reasons are stored in the Questions vector
		*/

		//$ we assume that nodelist.getLength()=1, since there can be
		//$ only one "explain" block
		org.w3c.dom.Node current=nodelist.item(0);
		String nodename=current.getNodeName();
		String nodevalue=current.getNodeValue();				
			
		//$ the #text part is the explanation
		if (nodename=="#text") {
			//$ if the nodevalue exists then trim it
			if (nodevalue != null) 
				nodevalue=nodevalue.trim();	

			//$ if the string contains data and it is text, it must be a answer
			if (nodevalue.length() > 0) {
				nodevalue=HtmlConvert(nodevalue);
				((Question) Questions.elementAt(question)).addFeedback(nodevalue);	
			}
		}		
	}

	public void getAnswer(org.w3c.dom.NodeList nodelist, int question,boolean correct) {
		/* post:  the answers are stored in the Questions vector
		*/
	
		// used to determine the current answer
	
		for (int i=0;i<nodelist.getLength();i++) {									
		
		//$ request one child and its value
		org.w3c.dom.Node current=nodelist.item(i);
			
		String nodename=current.getNodeName();
		String nodevalue=current.getNodeValue();									
			
		//$ first check if nodevalue really points to text
		if (nodename =="#text") {
	
			//$ if the nodevalue exists then trim it
			if (nodevalue != null) 
				nodevalue=nodevalue.trim();
	
			//$ if the string contains data and it is text, it must be a answer
			if (nodevalue.length() > 0) {				
	
				//$ if the answer isn't empty, it can be added to the list 
				nodevalue=HtmlConvert(nodevalue);		
				if (correct) 
					((Question) Questions.elementAt(question)).addGoodAnswer(nodevalue);
				else
					((Question) Questions.elementAt(question)).addBadAnswer(nodevalue);		
				((Question) Questions.elementAt(question)).addFeedback("");	
			}	 
		} else
			//$ if the nodename==answer then the child is the answer text
			if (nodename=="explain") {
				org.w3c.dom.NodeList reason=current.getChildNodes();
				getReason(reason,question,correct);
			}
		}
	}

	public void getQuestion(org.w3c.dom.NodeList nodelist,int question) {
		/* post: the questions are stored in one vector 
		*/

		int NbAnswers=0;
		int NbCorrect=0;

		//$ Analyse each of the nodes to see if they contain any
		//$ any questions or answers
		for (int i=0;i<nodelist.getLength();i++) {									
			//$ request one child and its value
			org.w3c.dom.Node current=nodelist.item(i);
			String nodename=current.getNodeName();
			String nodevalue=current.getNodeValue();									
		
			//$ first check if nodevalue really points to text
			if (nodename =="#text") {

				//$ if the nodevalue exists then trim it
				if (nodevalue != null) 
				nodevalue=nodevalue.trim();

				//$ if the string contains data and it is text, it must be a question
				if (nodevalue.length() > 0) {
			
					nodevalue=HtmlConvert(nodevalue);
					//$ if the question isn't empty, it can be added to the list 
				 	((Question) Questions.elementAt(question)).setQuestion(nodevalue);
				}	 
			} else
				//$ if the nodename==answer then the child is the answer text
				if (nodename=="answer") {
					org.w3c.dom.NodeList answer=current.getChildNodes();
					getAnswer(answer,question,((org.w3c.dom.Element)current).getAttribute("correct").equalsIgnoreCase("true"));
					NbAnswers++;

					if (((org.w3c.dom.Element)current).getAttribute("correct").equalsIgnoreCase("true"))
						NbCorrect++;	
				}
		}
		//$ save the number of answers and the number of correct ones
		((Question)Questions.elementAt(question)).setSize(NbAnswers, NbCorrect);
	}


	public void fillArrays(org.w3c.dom.Element root) throws AhaException {
		/* pre:root is the root element of the XML document
		post: The XML document has been parsed and the Question vector is
		filled 
		*/

		int question=0;
		//$ Save title,knowledge, TotalQuestions and Verbose in global variables
 
		try {
			Title=((org.w3c.dom.Element) root).getAttribute("title").trim();
			Knowledge=((org.w3c.dom.Element) root).getAttribute("concept").trim();		 	
			TotalQuestions=Integer.valueOf(((org.w3c.dom.Element) root).getAttribute("total").trim()).intValue();
			Verbose=((org.w3c.dom.Element) root).getAttribute("verbose").trim().equalsIgnoreCase("true");
	
			QuestionsToAsk=Integer.valueOf(((org.w3c.dom.Element) root).getAttribute("ask").trim()).intValue();

			if (TotalQuestions < 1)
				throw new AhaException("Invalid quiz file: number of questions (" + TotalQuestions + ") should be at least 1.");
			if (QuestionsToAsk < 1)
				throw new AhaException("Invalid quiz file: number of questions to ask (" + QuestionsToAsk + ") should be at least 1.");
			if (TotalQuestions < QuestionsToAsk)
				throw new AhaException("Invalid quiz file: number of questions to ask (" + QuestionsToAsk + ") is higher than total number of questions (" + TotalQuestions + ").");
		} catch (NumberFormatException e) {
			throw new AhaException("Invalid quiz file: non-number on line with number of answers to question.");
		}


		//$ Add a new question to the vector
		Questions.addElement(new Question());

		for (org.w3c.dom.Node nod = root.getFirstChild();
			nod != null;
			nod = nod.getNextSibling()) {
		 
			 //$ Analyse each of the Child nodes

			//$ select nodename
			String nodename=nod.getNodeName();
	
			//$ Check if a question block is started	
			if (nodename.equalsIgnoreCase("question")) {
				
			 	//$ Get all the child nodes (the question itself and answers)
				org.w3c.dom.NodeList nodelist=nod.getChildNodes();		

				//$ and add the questions to the vector
				getQuestion(nodelist,question);	
				Questions.addElement(new Question());
			
				((Question)Questions.elementAt(question)).verify();	

				((Question)Questions.elementAt(question)).setAnswersToShow(Integer.valueOf(((org.w3c.dom.Element)nod).getAttribute("answers").trim()).intValue());

				question++;

			} 
		}
		if (question != TotalQuestions) 
			throw new AhaException("Invalid quiz file: number of questions in file (" + question + ") does not match the expected number given at the top of the file (" + TotalQuestions + ").");
	}


	/**
	 * This method creates a random list of question numbers.
	 */
	public int[] GenerateQuestions() {
		int i;
		int [] qlist = new int[QuestionsToAsk];
		// we encode the answer indices into alist:
		// positive for good answers, negative (-1 -index) for bad answers
		boolean [] chosen = new boolean[TotalQuestions];
		for (i = 0; i < TotalQuestions; ++i)
			chosen[i] = false;
		R = new Random((long)UserModel.getId().hashCode());
		for (i = 0; i < QuestionsToAsk; ++i) {
			int j, y=0;
			do {
				j = RAND(QuestionsToAsk);
			} while (chosen[j] && ++y<10000);
			chosen[j] = true;
			qlist[i] = j;
			alist.addElement(BuildAnswers((Question)Questions.elementAt(j)));
		}
		return qlist;
	}

	/**
	 * This method creates a random list of answers to a question.
	 */
	public int[] BuildAnswers(Question question) {
		// position of good answers in output
		int [] goods = new int[question.getGoodAnswers().size()];
		// position of bad answers in output
		int [] bads = new int[question.getBadAnswers().size()];
		// slots to fill with answers
		int [] Answers = new int[question.getAnswersToShow()];
		int i;
		// make sure there are enough slots.
		i = question.getAnswersToShow();
		if (i < question.getGoodAnswers().size())
			i = question.getGoodAnswers().size();
		if (i < question.getBadAnswers().size())
			i = question.getBadAnswers().size();
		boolean [] slots = new boolean[i];

		for (i = 0; i < goods.length; ++i)
			goods[i] = -1;
		for (i = 0; i < bads.length; ++i)
			bads[i] = -1;
		for (i = 0; i < slots.length; ++i)
			slots[i] = false;
		// pick the desired number of good answers and place them randomly.
		for (i = 0; i < question.getRightToShow(); ++i) {
			int x, y=0;
			do
				x = RAND(question.getAnswersToShow());
			while (slots[x] && ++y<10000);
			slots[x] = true;
			goods[i] = x;
		}
		for (i = 0; i < question.getAnswersToShow()-question.getRightToShow(); ++i) {
			// pick the desired number of bad answers and place them randomly
			// in the remaining slots.
			int x, y=0;
			do
				x = RAND(question.getAnswersToShow());
			while (slots[x] && ++y<10000);
			slots[x] = true;
			bads[i] = x;
		}
		// now that we know where to place good and bad answers
		// we pick the answers from the available ones.
		for (i = 0; i < question.getGoodAnswers().size(); ++i)
			slots[i] = false;
		for (i = 0; i < question.getRightToShow(); ++i) {
			int x;
			do
				x = RAND(question.getGoodAnswers().size());
			while (slots[x]);
			slots[x] = true;
			Answers[goods[i]] = x;
		}
		for (i = 0; i < slots.length; ++i)
			slots[i] = false;
		for (i = 0; i < question.getAnswersToShow()-question.getRightToShow(); ++i)
			slots[i] = false;
		for (i = 0; i < question.getAnswersToShow()-question.getRightToShow(); ++i) {
			int x;
			do
				x = RAND(question.getBadAnswers().size());
			while (slots[x]);
			slots[x] = true;
			Answers[bads[i]] = -1 - x;
		}
		return Answers;
	}

	/**
	 * This method generates a multiple-choice quiz with randomly selected
	 * questions and answers.
	 */
	public void PrintQuestions(String QuizCgi) {
		int[] questions = GenerateQuestions();
		AHA.getMessages().PrintBodyTag(); /* prints BODY tag */
		out.println("<H1>" + Title + "</H1>");
		out.println("<FORM METHOD=\"POST\" ACTION=\"" + QuizCgi + "\">");
		out.println("<OL>");
		for (int i = 0; i < QuestionsToAsk; ++i) {
			out.println("<P><LI>");
			out.println(((Question)Questions.elementAt(questions[i])).getQuestion() + "<BR>");
			for (int j = 0; j < ((Question)Questions.elementAt(questions[i])).getAnswersToShow(); ++j) {
				int ai = ((int[])(alist.elementAt(i)))[j];
				if (((Question)Questions.elementAt(questions[i])).getRightToShow() > 1)
					out.println("<INPUT TYPE=\"CHECKBOX\" NAME=\"check_" + i + "_" + j + "\" VALUE=\"TRUE\">");
				else
					out.println("<INPUT TYPE=\"RADIO\" NAME=\"radio_" + i + "\" VALUE=\"radio_" + j + "\">");
				if (ai >= 0)
					out.println((String)(((Vector)(((Question)Questions.elementAt(questions[i])).getGoodAnswers())).elementAt(ai)));
				else
					out.println((String)(((Vector)(((Question)Questions.elementAt(questions[i])).getBadAnswers())).elementAt(-1 -ai)));
				out.println("<BR>");
			}
		}
		out.println("</OL>");
		out.println("<HR>");
		out.println("<INPUT TYPE=\"SUBMIT\" VALUE=\"Evaluate this Test\">");
		out.println("<INPUT TYPE=\"RESET\" VALUE=\"Wipe Answers to Try Again\">");
		out.println("</FORM>");
		AHA.getMessages().PrintEndBodyTag(); /* prints /BODY tag */
	}

	/**
	 * This method reads a user's answers to a multiple-choice quiz
	 * and evaluates the answers. The given user model is updated.
	 *
	 * @exception	AhaException	If an error occurs in decoding the user's
	 * answers to the quiz.
	 */
	public String EvalQuestions(CgiUtil Util) throws AhaException {
		int[] thequestions = GenerateQuestions();
		Vector [] questions = new Vector[QuestionsToAsk];
		int CorrectAnswersThisQuestion = 0;
		int ErrorsThisQuestion = 0;
		int CorrectAnswers = 0;
		int QuestionsAnswered;
		int Score=0;
		int i;
		String attribute;
		String value;
		for (i = 0; i < QuestionsToAsk; ++i)
			questions[i] = null;
		for (Enumeration e = Util.keys(); e.hasMoreElements(); ) {
			attribute = (String)e.nextElement();
			value = (String)Util.get(attribute);
			if (attribute.startsWith("check_")) {
				// checkbox, thus multiple answers possible
				int qnr = Integer.decode(attribute.substring(6, attribute.indexOf('_', 6))).intValue();
				int anr = Integer.decode(attribute.substring(attribute.lastIndexOf('_')+1)).intValue();
				if (questions[qnr] == null) {
					questions[qnr] = new Vector(((Question)Questions.elementAt(thequestions[qnr])).getAnswersToShow(), 1);
					for (i = 0; i < ((Question)Questions.elementAt(thequestions[qnr])).getAnswersToShow(); ++i)
						questions[qnr].addElement("");
				}
				questions[qnr].setElementAt("TRUE", anr);
			} else if (attribute.startsWith("radio_")) {
				// radio button, thus only one answer possible
				int qnr = Integer.decode(attribute.substring(6)).intValue();
				int anr = Integer.decode(value.substring(6)).intValue();
				if (questions[qnr] == null) { // for a radio button this should always be the case
					questions[qnr] = new Vector(((Question)Questions.elementAt(thequestions[qnr])).getAnswersToShow(), 1);
					for (i = 0; i < ((Question)Questions.elementAt(thequestions[qnr])).getAnswersToShow(); ++i)
						questions[qnr].addElement("");
				}
				questions[qnr].setElementAt("TRUE", anr);
			} else {
				// unrecognized form input; ignore (may be submit button or something)
				;
			}
		}
		// now we have read the forms input and are ready to evaluate.
		AHA.getMessages().PrintBodyTag(); /* prints BODY tag */
		out.println("<H1>" + Title + "</H1>");
		for (i = 0, QuestionsAnswered = 0; i < questions.length; ++i)
			if (questions[i] != null)
		++QuestionsAnswered;
		if (QuestionsAnswered < QuestionsToAsk) {
			out.println("In order to evaluate this test, you must answer all the questions.");
			out.println("You appear to have answered only " + QuestionsAnswered + " out of " + QuestionsToAsk + " questions.");
			out.println("<BR>\nPlease go back and try again.");
			return "0/" + QuestionsToAsk;
		} else {
			if (Verbose)
			out.println("<OL>");
			for (i = 0; i < QuestionsToAsk; ++i) {
				CorrectAnswersThisQuestion = 0;
				ErrorsThisQuestion = 0;
				if (Verbose) {
					out.println("<P><LI>");
					out.println(((Question)Questions.elementAt(thequestions[i])).getQuestion());
				}
				for (int j = 0; j < ((Question)Questions.elementAt(thequestions[i])).getAnswersToShow(); ++j) {
					if (((String)questions[i].elementAt(j)).equals("TRUE")) {
					if (Verbose) {
						out.println("<BR>Your answer:");
					}
					int ai = ((int[])(alist.elementAt(i)))[j];
					if (ai >= 0) {
						++CorrectAnswersThisQuestion;
					if (Verbose) {
						out.println((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getGoodAnswers())).elementAt(ai)));
						out.println("<BR>\nThis answer is <B>correct</B>.<BR>");
					}
					if (!((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getGoodFeedback())).elementAt(ai))).equals("")) {
						if (Verbose) {
							out.println("Additional comment:<BR>");
							out.println((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getGoodFeedback())).elementAt(ai)));
						}
					}
				} else {
					++ErrorsThisQuestion;
					if (Verbose) {
						out.println((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getBadAnswers())).elementAt(-1 -ai)));
						out.println("<BR>\nThis answer is <B>incorrect</B>.<BR>");
					}
					if (!((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getBadFeedback())).elementAt(-1 -ai))).equals("")) {
						if (Verbose) {
							out.println("Additional comment:<BR>");
								out.println((String)(((Vector)(((Question)Questions.elementAt(thequestions[i])).getBadFeedback())).elementAt(-1 -ai)));
						}
					}
				}
			}
		}
		if (ErrorsThisQuestion==0 && CorrectAnswersThisQuestion == ((Question)Questions.elementAt(thequestions[i])).getRightToShow())
			++CorrectAnswers;
		else
			if (ErrorsThisQuestion==0 && ((Question)Questions.elementAt(thequestions[i])).getRightToShow() > 1)
				if (Verbose)
					out.println("<BR>There are more correct answers. Therefore your answer to this whole question is considered to be <B>incorrect</B>.");
			}
			Score = (100 * CorrectAnswers) / QuestionsToAsk;
			if (Verbose) {
				out.println("</OL>");
				out.println("<HR>");
			}
			out.println("Your score: " + Score + "%.");
			if ((Score==100 || Verbose) && Knowledge!=null) {
				// add knowledge from this test to the user model
				UserModel.getKnowledge().put(Knowledge,"100");
			}
			if (!Verbose && (Score<100)) {
				out.println("<HR>\nYour score was not high enough. Please go back and try again.");
			}
		}
		AHA.getMessages().PrintEndBodyTag(); /* prints /BODY tag */
		return CorrectAnswers + "/" + QuestionsToAsk;
	}

	private int RAND(int limit) {
		return (int)(R.nextFloat() * limit) % limit;
	}

	public String FilterLinks(String Link) { return Link;}
}
