/*
 * @(#)Generate.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.util.StringTokenizer;
import java.util.Hashtable;

/**
 * The class Generate is used for generating knowledge for a user.
 * The GenerateList class provides the knowledge generation clauses.
 */

public class Generate{
	private int perc;
	private String name;
	private AHA AHA;

	// ceil takes care off rounding of doubles.
	// ceil = true for a "Generates_rest" of "Generates_full" and false otherwise
	// if ceil then round up, else down

	private boolean ceil;
	private boolean debug = false;

	public void println(Object s) {if (debug) System.out.println(s.toString());}
	public void print(Object s) {if (debug) System.out.print(s.toString());}

	public Generate(AHA AHA, String name, int perc,boolean ceil) throws AhaException{
		this.AHA = AHA;
		this.name = name;
		this.perc = perc;
		this.ceil = ceil;
			// lift yourself with perc points
		updateKnowledge();
			// lift children with perc percent
		updateChildren();
	}

	/**
	 * Update knowledge of this concept (name)
	 */
	public void updateKnowledge() {
		println("Updating knowledge of "+name+" with "+perc+"%.");
		Hashtable Knowledge = AHA.getUserModel().getKnowledge();
		if (Knowledge.containsKey(this.name)) {
			String oldperc = (String)Knowledge.get(name);
			int oldp = Integer.parseInt(oldperc);
			Integer newperc = new Integer(oldp+perc);

			// Check for newperc <= 100 and =>0
			if (newperc.intValue()<0) newperc = new Integer(0);
			if (newperc.intValue()>100) newperc = new Integer(100);
			Knowledge.put(name,newperc.toString());
		} else {	// knowledge was zero
			Knowledge.put(name,new Integer(perc).toString());
		}
		println("New knowledge for "+name+" = "+ (String)Knowledge.get(name));
	}

	/**
	 * Update knowledge of the children of this concept (name)
	 */
	public void updateChildren() throws AhaException{
		// read generate from file
		GenerateList gl = AHA.getGenerateList();
		if (gl.containsKey(name)) {
			String genlist = (String)gl.get(name);
	//		println("Update children: "+genlist);

			// now parse the concepts in genlist
			StringTokenizer st = new StringTokenizer(genlist);
			while (st.hasMoreTokens()) {
				String n = st.nextToken();
				int p = n.indexOf(":");
				if (p>-1) {	// parse number en zo
					String nname = n.substring(0,p);
					String action = n.substring(p+1);
				//	print(nname+" "+action+". ");
					int number = 0;;
					try {
						if (action.startsWith("+")) {
							number = Integer.parseInt(action.substring(1));
						} else {	// number is negative or has no sign
							number = Integer.parseInt(action);
						}
					} catch (NumberFormatException nfe) {
						println("The number cannot be parsed: "+action);
					}
					double doubleperc = perc;
					double doublenumber = number;
					double doublep = (doubleperc / 100) * doublenumber;
					int liftperc = roundDouble(doublep);
				//	println("high="+(int)Math.ceil(doublep)+ "  low="+(int)Math.floor(doublep));
				//	println("Lift percentage of "+nname +": "+liftperc);
					// test: don't recursive generate knowledge for self
					if (!nname.equals(name)) {
						Generate gchild = new Generate(AHA,nname,liftperc,this.ceil);
					} else {
					//	println("Don't update self again");
					}
				} else {
					println("ERROR no generate action: "+n);
					throw new AhaException("Internal Error: Generate knowledge fires unknown action");
				}
			}
		} else {
			// end of fire tree leaf
				println("I tried to fire on "+name+", but it has no generates-list");
		}
	}

	/**
	 * roundDouble returns the ceiling of floor of a double, depending
	 * on the boolean "ceil" field.
	 */
	public int roundDouble(double d) {
	
		if (this.ceil) return (int)Math.ceil(d);
		else return (int)Math.floor(d);
	}
}