/*
 * @(#)Get.java	1.0	98/07/28
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.net.URL;
import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * A servlet for processing normal page requests for the AHA adaptive
 * hypermedia engine.
 *
 */


public class Get extends AhaServlet {
	private boolean debug = true;
	public void println(Object s) { if (debug) System.out.println(s.toString());}
	public void print(Object s) {if (debug) System.out.print(s.toString());}
	public void println(HttpServletRequest req) {
		System.out.println(req.getMethod()+" "+req.getRequestURI());
	}

	Hashtable UserModelNames;

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		UserModelNames = new Hashtable(100, (float)0.1);
	}

	/**
	 * The method doGet handles GET requests.
	 * GET requests are used for:
	 * <UL>
	 * <LI> handling requests for pages
	 * <LI> generating the list of pages that were read
	 * <LI> generating the list of pages still to be read
	 * <LI> generating multiple-choice tests
	 * </UL>
	 */

	private int counter = 0;

	public void service(HttpServletRequest request,
				HttpServletResponse response) {

			counter++;

		System.out.println();
		System.out.println(counter);
		Timer t1 = new Timer();

		HttpSession session = request.getSession(false);
	if (session!=null) synchronized(session.getId()) {
		println("synchronized{");

		println("-------- Initializing information Request ---------");
		println(request);

		try {
			super.initService(request,response);
		} catch (AhaException ahae) {
			System.out.println(getName()+": "+ahae.toString());
			ahae.printStackTrace();
			super.redirect(request, response);
			return;
		}


		try {
			Messages Messages = AHA.getMessages();
			String URI = request.getRequestURI();
			// read the Generate-actions some of which will be processed
				// We first do a "sanity check" to see whether the servlet was initialized
				// properly (i.e. whether the required fields are set).
				if (HtDocs == null)
				throw new AhaException("The AHA Get is not initialized properly. The HTDOCS value is not set. Please notify the author that this error occurred.");
				if (Course == null)
					throw new AhaException("The AHA Get is not initialized properly. The COURSE value is not set. Please notify the author that this error occurred.");
				if (Logs == null)
					throw new AhaException("The AHA Get is not initialized properly. The LOGS value is not set. Please notify the author that this error occurred.");
				if (Title == null)
					throw new AhaException("The AHA Get is not initialized properly. The TITLE value is not set. Please notify the author that this error occurred.");
				if (Background == null)
					throw new AhaException("The AHA Get is not initialized properly. The BACKGROUND value is not set. Please notify the author that this error occurred.");
				if (Language == null)
					Language = "English"; // the default language;

				int temp = URI.indexOf('/',0);
				if (temp < 0)
					throw new AhaException("This request doesn't contain a name of a page. Please notify the author.");
				String Id = loginid;
				String DocName = new Derive(URI).getDoc();

				// Now we can start writing the html part.
				// As soon as we request the response's Writer the header is sent
				// and cannot be altered anymore.
	
				// In order to initialize the User Model we need the filename
				// of the logfile.

				String LogName = Logs.concat("/" + Course + "/" + Id);
				int NbPages = 0; //DocList.size();
				int NbRead = 0;
				String FullDocName = null;
				// We make the LogName a unique object that can be used for locking.
				if (!UserModelNames.contains(LogName))
					UserModelNames.put(LogName,LogName);
				LogName = (String)UserModelNames.get(LogName);

				FullDocName = HtDocs + "/" + Course + "/xml/" + DocName;
				DocName = "Get/" + DocName;
				// for the relevant cases we try to read the requested document.
				try {
					File docfile = new File(FullDocName);
					if (false) //(!docfile.canRead())
						throw new AhaException("Cannot access document '" + FullDocName +
						"' on the server.\nIf this error is the result of following a link,\n" +
						"please notify the author that this error occurred.");
				} catch (NullPointerException e) {
						throw new AhaException("Name of requested document appears to be null. Please notify the author that this error occurred.");
				} catch (Exception e) {
					throw new AhaException("Internal Error: An unknown exception occured reading file '" + FullDocName + "' on the server.\nPlease notify the author that this error occurred.");
				}

				// Find out about read and still to read pages.
				Enumeration enum = UserModel.getVisitedList().keys();
				while(enum.hasMoreElements()) {
					String key = (String)enum.nextElement();
					String value = (String)UserModel.getVisitedList().get(key);
					if (value.equals("100")) 
					++NbRead;
				}
				UserModel.setNbRead(NbRead);
				UserModel.setNbPages(NbPages);
	

				println("Filtering "+FullDocName);
				// fill in document
				if (DocName.equals("Get/")) {
					Messages.Fatal("Error: a page name must be provided.");
					return;
				} 


				File xmlfile = new File(FullDocName);
				if (!xmlfile.canRead()) {
					throw new AhaException("Error: document "+FullDocName+" does not exist.");
				} else new PageRequestHandler(AHA,FullDocName,DocName).run();

				Messages.PrintSimpleFooter();
				out.flush();

				// add information to logfile
				// deleted code generates printable date info, which is not needed.
				Date date = Calendar.getInstance().getTime();
				UserModel.getLogFileLines().addElement(DocName + " " + date.getTime()/1000 /* + " " + datestr */);
				UserModel.SaveUserModel(LogName);

				// put the updated usermodel in the session for the next access
				//session.putValue("usermodel", UserModel);

				// Theres is no need to do this, right?
				AHA.setUserModel(UserModel);
				println("Usermodel saved");

				out.close();
			} catch (Exception e) {
				println(getName() + ": " + e.toString());
				// e.printStackTrace();
				try {
					out = response.getWriter();
				} catch (IOException ioe) {
					System.out.println(ioe.toString());
				}
				out.println("AHA Error: "+e.toString());
			}
			println("}");
		}
		else {
			// session = null
			System.out.println(getName()+" : No Session.");
			super.redirect(request, response);
		}
		Timer t2 = new Timer();
	//	System.out.print("Get tijd:" );
	//	System.out.println(t2.getTime() - t1.getTime());
		System.gc();
	}

}
