package AHA;

import XmlUtils.*;
import Expression.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.*;

public class HeaderFooterFilter {
	private String filename;
	private AHA AHA;
	private StringBuffer buffer;
	private Element el;

	private String DocName;

	private boolean debug = true;
	private void println(Object s) {if (debug) System.out.println(s.toString());}

	private final String servletcode = "servlet";
	private final String variablecode = "variable";
	private final String authordefinedHTML = "#text";
	private final String stopapplet = "stopapplet";

	public HeaderFooterFilter(AHA AHA, String filename, String DocName) throws AhaException{
		this.AHA = AHA;
		this.filename = AHA.getHtDocs() + "/" + AHA.getCourse() + "/" + filename;
		this.DocName = DocName;
		this.buffer = new StringBuffer();
	
		XmlReader reader;
		try {
			reader  = new XmlReader(this.filename);
		} catch (Exception e) {
			throw new AhaException("Fatal error while reading "+filename);
		}
		this.el = reader.openFile();
	}



	public StringBuffer makeHTML() throws AhaException {
		buffer = filterChildNodes(el);
		return buffer;
	}

	private StringBuffer filterChildNodes(Node nodec) throws AhaException{
		StringBuffer sb = new StringBuffer();
		NodeList nodelist = nodec.getChildNodes();
		for (int i=0; i<nodelist.getLength(); i++) {
			Node node = nodelist.item(i);
			if (!node.toString().trim().equals(""))
				sb.append(filterNode(node));
		}
		return sb;
	}		

	private StringBuffer filterNode(Node nodec) throws AhaException{
		StringBuffer sb = new StringBuffer();
		String name = nodec.getNodeName().trim();
		String value = nodec.getNodeValue();
		if (!nodec.toString().trim().equals("")) {
			if (name.equals(servletcode)) {
				sb.append(servletLink(nodec));
			} else if (name.equals(variablecode)) {
				sb.append(variable(nodec));
			} else if (name.equals(stopapplet)) {
				sb.append(stopAppletCode(nodec));
			} else {
				println("HTML{");
				sb.append(HTMLcode(nodec));
				println(sb); println("}");
			}
		}
		return sb;
	}

	public StringBuffer stopAppletCode(Node node) {
		StringBuffer sb = new StringBuffer();
		sb.append("<applet code=\"Stop.class\" codebase=\"http://"
			+ AHA.getHost() + ":" + AHA.getPort() + "/" + AHA.getCourse()
			 + "/" + AHA.getApplets() + "\" + width=\"2\" height=\"2\">\n");
		String stopurl = "http://" + AHA.getHost() + ":" + AHA.getPort() 
			+ AHA.getServletDir() + "/Stop/" + AHA.getUserModel().getId() + "/" + DocName.substring(4) + "/";
		String stopvalue = AHA.getHtDocs() + "_" + AHA.getCourse() + "_" + AHA.getLogs() + "_" 
			+ AHA.getTitle() + "_" + AHA.getBackground() + "_" + AHA.getLanguage();
		String stopurlvalue = stopurl +java.net.URLEncoder.encode(stopvalue);
		sb.append("<param name=\"STOPURL\" value=\""+stopurlvalue + "\">");
		sb.append("<param name=\"HOST\" value=\"" + AHA.getHost() + "\">\n");
		sb.append("<param name=\"PORT\" value=\"" + AHA.getPort() + "\">\n");
		sb.append("</applet>\n");
		return sb;
	}

	public StringBuffer HTMLcode(Node node) throws AhaException {
		StringBuffer sb = new StringBuffer();
		sb.append(node.toString());
		return sb;
	}
	public StringBuffer variable(Node node) throws AhaException {
		StringBuffer sb = new StringBuffer();
		String var = node.getFirstChild().getNodeName();
		sb.append(deriveVariable(var));
		return sb;
	}
 
	public String deriveVariable(String dtdvar) throws AhaException {
		String name = "";
		if (dtdvar.equals("username"))
			name = AHA.getUserModel().getName();
		if (dtdvar.equals("loginid"))
			name = AHA.getUserModel().getId();
		if (dtdvar.equals("course"))
			name = AHA.getCourse();
		if (dtdvar.equals("title"))
			name = AHA.getTitle();
		if (dtdvar.equals("numberdone")) {
			int p = AHA.getUserModel().getNbGoodVisited();
			name = new Integer(p).toString();}
		if (dtdvar.equals("numbertodo")) {
			int p = AHA.getUserModel().getNbPages() - AHA.getUserModel().getNbGoodVisited();
			name = new Integer(p).toString();}
		if (name.equals(""))
			throw new AhaException("Unknown variable: "+dtdvar);
		return name;
	}


	public StringBuffer servletLink(Node node) throws AhaException {
		StringBuffer sb = new StringBuffer();
		String servletdir = AHA.getServletDir();
		String Id = AHA.getUserModel().getId();
		String Course = AHA.getCourse();
		String servletname = deriveServletName(node.getFirstChild().getFirstChild().getNodeName());
		String href = servletdir + "/" + servletname + "/" + Course + "/" + Id;
		StringBuffer title = filterChildNodes(node.getLastChild());
		//String title = node.getLastChild().toString();


		// build the HTML code of the anchor tag with the link to the servlet
		sb.append("<a href=\"");
		sb.append(href);
		sb.append("\">");
		sb.append(title);
		sb.append("</a>");
		return sb;
	}

	public String deriveServletName(String dtdname) throws AhaException {
		String name = "";
		if (dtdname.equals("logout")) 
			name = "Logout";
		if (dtdname.equals("colorsettings"))
			name = "ColorConfig";
		if (dtdname.equals("knowledgesettings")) 
			name = "KnowledgeConfig";
		if (dtdname.equals("passwordsettings")) 
			name = "PasswordConfig";
		if (dtdname.equals("done"))
			name = "Done";
		if (dtdname.equals("todo"))
			name = "Todo";
		if (name.equals("")) 
			throw new AhaException("Unknown servlet");
		return name;
	}
}		