
/*
 * @(#)Messages.java	1.0	98/07/31
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

	/**
	 * Messages contains common routines for creating headers, footers,
	 * and error messages.
	 */

public class Messages {
	private boolean debug = false;
	private AHA AHA;
	private String historyline; // the header line with info on the read pages.


	UserModel UserModel;
	String HtDocs;
	String Course;
	String Logs;
	String Title;
	String Background;
	int Language;
	String Icons;
	String Host;
	int Port;
	String Applets;
	PrintWriter out;
	String Id;


	private void println(Object s) {if (debug) System.out.println(s.toString());}
	private void print(Object s) {if (debug) System.out.print(s.toString());}

	/**
	 * The default constructor does nothing.
	 * It is not allowed but at this stage nothing is set up to throw exceptions yet.
	 */
	public Messages() {
	}

	/**
	 * The standard constructor only initializes the AHA context.
	 * All other fields must be set later.
	 */
	public Messages(AHA AHA) {
		this.AHA = AHA;
		UserModel = AHA.getUserModel();
		HtDocs = AHA.getHtDocs();
		Course = AHA.getCourse();
		Logs = AHA.getLogs();
		Title = AHA.getTitle();
		Background = AHA.getBackground();
		Language = AHA.getLanguage();
		Icons = AHA.getIcons();
		Host = AHA.getHost();
		Port = AHA.getPort();
		Applets = AHA.getApplets();
		out = AHA.getWriter();
		Id = UserModel.getId();
	}

	/**
	 * This method generates a minimal HTML header with a title for an
	 * error message.
	 */
	public void PrintSimpleHeader() {
	  // Generate required part the HTML header in case something
	  // went wrong, so rely on as little as possible.
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<HTML>");
		out.println("<HEAD>");
		out.println("<TITLE>An Error Occurred.</TITLE>");
		out.println("</HEAD>");
		out.println("<BODY BGCOLOR=\"#FFFFFF\">");
	}

	/**
	 * This method closes the HTML file's BODY and HTML tags.
	 */
	public void PrintSimpleFooter() {
		out.println("</HTML>");
	}

	/**
	 * This method generates the required header for every HTML page.
	 * It does not show visible output in the BODY of the document.
	 * It needs the user model to initialize the style sheet with link colors.
	 * The Course, HtDocs, Icons and Background are needed to set the background
	 * image. The Title is needed in the header.
	 * This method is only called by AhaServlet, because it must be done
	 * for all servlets.
	 */
	public void InitHeader() {
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<HTML>");
		out.println("<HEAD>");
		out.println("<TITLE>" + Title + "</TITLE>");
		out.println("<STYLE TYPE=\"text/css\">");
		out.println("A.Good { color: " + UserModel.printGoodLinkColor() + " }");
		out.println("A.Bad { color: " + UserModel.printBadLinkColor() + " }");
		out.println("A.Neutral { color: " + UserModel.printNeutralLinkColor() + " }");
		out.println("A.Unconditional { color: " + UserModel.printNeutralLinkColor() + " }");
		out.println("</STYLE>");
		//String baseurl = "http://"+Host+":"+Port+AHA.getBase()+"/";
		//out.println("<base href=\""+baseurl+"\">");
		out.println("</HEAD>");
	}

	/**
	 * This method generates the body tag in case we do not wish to
	 * print a header.
	 */
	public StringBuffer PrintBodyTag() {
		StringBuffer sb = new StringBuffer();
		sb.append("<BODY BACKGROUND=\"/" + Course + "/" + Icons + "/" +
			Background + "\" LINK=" +
			UserModel.printExternalLinkColor() + " ALINK=" +
			UserModel.printActiveLinkColor() + " VLINK=" +
			UserModel.printExternalVisitedColor() + ">"
		);
		return sb;
	}

	/**
	 * This method generates the author-defined header, and a standard part
	 * that contains "history" information.
	 * The author-defined part is filtered: wherever "ID" occurs it is replaced
	 * by the user's id. It is thus possible to include user dependent links
	 * in the header.
	 */

	// OVERBODIG

	public void PrintHeader(String headerfilename, boolean print) {
		if (print) 
			PrintBodyTag();
		PrintHeaderFile(headerfilename);
	}



	/**
	 * This method reads a headerfile and dumps it to out
	 * When the string "ID" is encountered then ID will be
	 * replaced with the user's id.
	 * When the string COURSE is encountered then COURSE will be
	 * replaced with the current course.
	 */

	// OVERBODIG
	public void PrintHeaderFile(String headerfilename) {
		String headerfile = headerfilename;
		if (headerfilename == null) {
			//headerfile = "header";
			return;
		}

		// Copy course dependent header.
		try {
			String line;
			String HeaderName = HtDocs + "/" + Course + "/" + headerfile;

			File file = new File(HeaderName);
			if (!file.canRead()) {
				out.println("<H2>Error: headerfile ("+HeaderName+")cannot be read from server. Please notify the author about this error.</H2>");
				System.out.println("Error: headerfile cannot be read from server.");
				return;
			}

			String Id = UserModel.getId();
			String Course = AHA.getCourse();
			BufferedReader HeaderFile = new BufferedReader(new FileReader(HeaderName), 1024);
			while ((line = HeaderFile.readLine()) != null) {
				int i = line.indexOf("/servlet/AHA/");
				if (i > -1) {
					i = i  + "/servlet/AHA/".length();
					// This is a call to a servlet, now add the Course & user's Id to the href link
					i = line.indexOf("\"", i);
					String begin = line.substring(0, i);
					String last = line.substring(i);
					line = begin + "/" +Course + "/"+ Id + last;
					// jump to possible other servlet links on the same line
					i = line.indexOf("/servlet/AHA/", i);
				}	

				out.println(line);
			}
			HeaderFile.close();
		} catch (IOException e) {
			System.out.println("Messages: IOException :" + e.toString());
			out.println("<H2>Error generating Standard Header!</H2>");
		}
	}

	/**
	 * Wordt niet meer gebruikt
	 */

	public void PrintStatus() {
		// Now print the standard part of the Header.
		switch (Language) {
		case AHA.Dutch:
			out.println("<B>" + UserModel.getName() + " heeft (" + UserModel.getNbGoodVisited()+","+UserModel.getNbBadVisited()+ ")/" + UserModel.getNbPages() + " pagina's gelezen.</B>");
			out.println("(<A HREF=\"http://" + Host + ":" + Port + AHA.getServletDir() + "/Done>deze gelezen</A>");
			out.println("<A HREF=\"http://" + Host + ":" + Port + AHA.getServletDir() + "/Todo>deze nog te doen</A>.)\n<HR>");
			break;
		case AHA.English:
		default:
			String liner = "<B>" + UserModel.getName() + " has read (" + UserModel.getNbGoodVisited() +","+UserModel.getNbBadVisited()+ ")/" + UserModel.getNbPages() + " pages.</B>";
			out.println(liner);
			liner = "(<A HREF=\"http://" + Host + ":" + Port + AHA.getServletDir() + "/Done\">these read</A>,";
			out.println(liner);
			liner ="<A HREF=\"http://" + Host + ":" + Port + AHA.getServletDir() + "/Todo\">these still to do</A>.)\n<HR>";
			out.println(liner);
			break;
		}
	}


	/**
	 * This method generates a /body tag, for the case where no visible
	 * footer is desired.
	 */
	public StringBuffer PrintEndBodyTag() {
		return new StringBuffer("</BODY>");
	}

	/**
	 * This method generates a footer, consisting of a standard part and an
	 * author-defined part.
	 * The standard part includes the Stop applet, used for determining when
	 * the user stops reading a page.
	 */

	// OVERBODIG
	public void PrintFooter(String DocName, String footerfilename, boolean print) {
		PrintFooterFile(DocName, footerfilename);
		if (print)
			PrintEndBodyTag();
	}

	/**
	 * This method returns the contents of a footerfile.	
	 * If one of the lines is equal to <applet> then that line will be replaced
	 * by a call to the Stop applet, used for determining when the user stops reading a page, 
	 * with the appropriate parameters. 
	 */

	// OVERBODIG
	public void PrintFooterFile(String DocName, String footerfilename) {

		String footerfile = footerfilename;
		if (footerfilename == null) { 
			//footerfile = "footer"; 
			return;
		}
		try {
			String line;
			String FooterName = HtDocs + "/" + Course + "/" + footerfile;
			File file = new File(FooterName);
			if (!file.canRead()) {
				System.out.println("Error: footerfile ("+FooterName+") cannot be read from server.");
				out.println("<H2>Error: footerfile ("+FooterName+") cannot be read from server.</H2>");
				return;
			}
			BufferedReader FooterFile = new BufferedReader(new FileReader(FooterName), 1024);
			while ((line = FooterFile.readLine()) != null) {
				if (line.trim().equals("<stopapplet>")) {
					out.println("<APPLET CODE=\"Stop.class\" CODEBASE=\"http://" + Host + ":" + Port + "/" + Course + "/" + Applets + "\" width=\"2\" height=\"2\">");
					String stopurl = "http://"+Host+":"+Port+ AHA.getServletDir() + "/Stop/"+Course+"/"+Id+"/"+DocName.substring(4)+"/";
					String stopvalue = HtDocs+"_"+Course+"_"+Logs+"_"+Title+"_"+Background+"_"+Language;
					String stopurlvalue = stopurl +java.net.URLEncoder.encode(stopvalue);
					println("Messages: stopurlvalue="+stopurlvalue);
					out.println("<PARAM name=STOPURL value=\""+stopurlvalue + "\">");
					out.println("<Param name=HOST value=\"" + Host + "\">");
					out.println("<Param name=PORT value=\"" + Port + "\">");
					out.println("</APPLET>");
				} else {

					int i = line.indexOf("/servlet/AHA/");
					if (i > -1) {
						i = i  + "/servlet/AHA/".length();
						// This is a call to a servlet, now add the Course & user's Id to the href link
						i = line.indexOf("\"", i);
						String begin = line.substring(0, i);
						String last = line.substring(i);
						line = begin + "/" +Course + "/"+ Id + last;
						// jump to possible other servlet links on the same line
						i = line.indexOf("/servlet/AHA/", i);
					}
					out.println(line);
				}
			}
			FooterFile.close();
		} catch (IOException e) {
			out.println("<P>Error reading footer file: " + e.toString());
		}
	}

	/**
	 * This method generates an HTTP relocation header.
	 * It may be used by the CGI version but not by the Servlet version.
	 */
	public void Relocation(String Url) {
		// Generate a relocation header, so the browser will load the
		// correct first document.
		out.println("Location: " + Url);
		out.println();
	}

	/**
	 * This method prints out an error message (after a minimal header).
	 */
	public void Error(String message) {
		PrintSimpleHeader();
		out.println("<H2>Error Occurred</H2>");
		out.println(message);
	}

	/**
	 * This method prints out a fatal error message, after a minimal header.
	 */
	public void Fatal(String message) {
		PrintSimpleHeader();
		out.println("<H2>Fatal Error Occurred</H2>");
		out.println(message);
		PrintSimpleFooter();
	}

	public static void dumpFatal(PrintWriter out, String message) {
	  // Generate required part the HTML header in case something
	  // went wrong, so rely on as little as possible.
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<HTML>");
		out.println("<HEAD>");
		out.println("<TITLE>An Error Occurred.</TITLE>");
		out.println("</HEAD>");
		out.println("<BODY BGCOLOR=\"#FFFFFF\">");		
		out.println("<H2>Fatal Error Occurred</H2>");
		out.println(message);
		out.println("</HTML>");
	}
}
