/**
 * @(#)PageRequestHandler.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;
import Expression.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * The class PageRequestHandler takes care of checking/generating
 * knowledge and calling the XmlFilter for a page.
 */

public class PageRequestHandler{
	private int full = 100;
	private int part = 35;
	private final int rest = full-part;
	private AHA AHA;
	private String FullDocName;
	private String DocName;
	private boolean debug = false;
	private String name;
	public void println(Object o) {if (debug) System.out.println(o.toString());}
	public void print(Object o) {if (debug) System.out.print(o.toString());}
	public PageRequestHandler(AHA AHA, String FullDocName, String DocName) throws AhaException{
		this.AHA = AHA;
		this.FullDocName = FullDocName;
		this.DocName = DocName;
		println("PageRequest: "+FullDocName);
		//println("DocName: "+DocName);
		int p = FullDocName.lastIndexOf("/");
		int q = FullDocName.lastIndexOf(".xml");
		println("p="+p+" q="+q);
		if (p>=q) {
			String error = "Due to a page-naming conventions the requested page is not valid.";
			if (FullDocName.indexOf(".xml") == -1) 
				error += " The file extension should be \".xml\". Please notify the author of the course.";
			throw new AhaException(error);
		}
		// the requested pagename is OK
		name = FullDocName.substring(p+1,q);
	}

	/**
	 * Do the job.
	 */
	public void run() throws AhaException{
		println("updating knowledge");
		// Update the user's knowledge
		CheckAndUpdateKnowledge();
		println("Knowledge updated.");
		// Now get the XML document
		new XmlFilter(AHA,FullDocName,DocName).run();
		Hashtable h = AHA.getUserModel().getVisitedList();
		Enumeration e = h.keys();
		while (e.hasMoreElements()) {
			String key = (String)e.nextElement();
			if (!FulfillRequirements(key)) {
			}
		}
	}

	private void CheckAndUpdateKnowledge2() throws AhaException{
		// this is a test case for the updating process
		// Generate_part . Generate_rest must work well for the whole tree
		System.out.println("Generate part");
		Generate g = new Generate(AHA,name,part,false);
		System.out.println("Generate rest");
		g = new Generate(AHA,name,rest,true);
	}	

	public void CheckAndUpdateKnowledge() throws AhaException{
		boolean b = FulfillRequirements(this.name);
		println("Fulfill: "+b);
		UserModel UserModel = AHA.getUserModel();
		String clause ="";
		if (AHA.getGenerateList().containsKey(name)) {
			clause = AHA.getGenerateList().get(name).toString();
		}
		println("Generate clause = "+clause+" :  "+isSwitch(name,clause));
		if (b) {
			// println("The user fulfills the requirements.");
			String requirement = "true";
			if (AHA.getRequirementList().containsKey(name)) {
				requirement = AHA.getRequirementList().get(name).toString();
			}
			println("Requirements met: "+requirement);
			if (UserModel.hasNotVisit(name)) {
				//	FULL
				//println("The user has not visited this page before.");
				//println("This visit will generate full knowledge.");
				println("Generate full");

				Hashtable vl = UserModel.getVisitedList();
				//println("-"+vl.toString());

				
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(full).toString());
				} else {
					println("This is a switch.");
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);
				println(UserModel.getVisitedList().toString());
				Generate g = new Generate(AHA,name,full,true);
			} else if (UserModel.hasBadVisit(name)) {
				//	REST
				//println("The user has visited this page before, but was not ready then.");
				//println("This visit will complete to full knowledge.");
				println("Generate rest");
				Hashtable vl = UserModel.getVisitedList();
				//println("-"+vl.toString());
				// i don't add the rest, but just put 100% instead
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(full).toString());
				} else {
					println("This is a switch.");
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);
				println(UserModel.getVisitedList().toString());

				Generate g = new Generate(AHA,name,rest,true);
				// set good visit
				// generate_rest
			} else if (UserModel.hasGoodVisit(name)) {
				//	NOTHING
				println("Nothing: you already visited this page");
					//println("The user has already visited this page.");
				//println("This visit will not change knowledge.");
				// generate nothing
			}				
		} else {
			println("Requirements not met: "+AHA.getRequirementList().get(name).toString());
			//println("The user has not enough knowledge yet for this concept.");
			if (UserModel.hasNotVisit(name)) {
				//	PART
				println("Generate part");
				// generat_part
				Hashtable vl = UserModel.getVisitedList();
				//println(vl.toString());
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(part).toString());
				} else {
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);
				println(UserModel.getVisitedList().toString());
				Generate g = new Generate(AHA,name,part,false);

			} else if (UserModel.hasBadVisit(name)) {
				//	NOTHING
				//println("This will not contribute to knowledge");
			} else if (UserModel.hasGoodVisit(name)) {
				//	? remove knowledge?
				println("First the user had a GOOD visit for "+name+", and now a BAD visit?");
				// what to generate now?
			}
		}

		// now update the pagecounters (pages (not,bad,good)read, total pages) in UserModel
		UserModel.recount();
	}

	public boolean isSwitch(String name, String clause) {
	// returns true if the clause for name contains name:0
		if (clause == null) return false;
		else return clause.indexOf(name+":-100")>-1;
	} 

	public boolean FulfillRequirements(String name) {
	// does the user fulfill the requirements for page name?
		RequirementList cr = AHA.getRequirementList();
		boolean b=false;
		if (!cr.containsKey(name)) { b = true;
		//	System.out.println("No requirements");
		}
		else {
			Expr expr = (Expr)cr.get(name);
			//System.out.print(name+ " "+expr.toString()+" ");
			b = expr.eval(AHA.getUserModel().getKnowledge());
		}
		return b;
	}
}
	