/*
 * @(#)UserModel.java	1.0	98/08/03
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import XmlUtils.*;
import java.io.*;
import java.util.*;
import java.awt.Color;
import org.w3c.dom.*;
import com.sun.xml.tree.*;

/**
 * 
 * UserModel represents the user model and maintains a permanent
 * version in the user's user model file and logfile.
 *
 */
public class UserModel implements Serializable { 

	public String toString() {
		return Name+" "+Email+" "+Id+" "+Passwd;
	}
	private boolean debug = false;

  private AHA AHA; // my context.
  private int nb_read; // number of pages read
  private int nb_to_read; // total number of pages to read

  private String Name="";    // name of the user
  private String Email="";        // email address of the user
  private String Id="";           // unique user id
  private String Passwd="";       // user's password as encoded by browser
  private String Institute="";    // institute of the user
  private String Department="";   // department or major of the user

  // The following colors are the default values used for different types of links.
  private Color GoodLinkColor=Color.blue; // color for desired links
  private Color BadLinkColor=new Color(32,32,32);  // color for undesired links
  private Color NeutralLinkColor=Color.magenta.darker().darker(); // color for uninteresting links
  private Color ExternalLinkColor=Color.red; // color for new external links
  private Color ExternalVisitedColor=new Color(127,0,0); // color for external visited links
  private Color ActiveLinkColor=Color.gray; // color for active links 

  private Hashtable Knowledge=new Hashtable(); // stores the concepts the user knows
	private Hashtable VisitedList = new Hashtable();


  
  private Vector LogFileLines; // stores the whole log info line by line
	private int NbNotVisited = -1;
	private int NbBadVisited = -1;
	private int NbGoodVisited = -1;

	private int NbRead = 0; // number of read pages.
	private int NbPages = -1; // number of pages to read.

  /** 
   * The default constructor is forbidden.
   *
   * @exception	AhaException	Always thrown because this constructor is not valid.
   */
  public UserModel()
       throws AhaException
  {
    throw new AhaException("Invalid call to default UserModel constructor. Please notify the author that this error occurred.");
  }

  /**
   * This is the "short" constructor, with one parameter: the AHA context.
   */
  public UserModel(AHA AHA) {
    this.AHA = AHA;
  }

	public void print(Object s) {if (debug) System.out.print(s.toString());}
	public void println(Object s) {if (debug) System.out.println("UserModel: "+ s.toString());}

	public UserModel(AHA AHA, String ModelName) throws AhaException {
		this(AHA);

		FileName filename = new FileName(ModelName, "model");

		File modelfile = new File(filename.toString());
		if (modelfile == null || !modelfile.exists()) 
			throw new AhaException("The File with your User Model does not exist. Please notify the author that this error occurred."+ModelName);
		synchronized(ModelName) {
		try {
			XmlReader xml = new XmlReader(filename.toString());
			Element el = xml.openFile();
			//println(el.toString());
			Node person = el.getChildNodes().item(0);
			this.Name = Utils.getFirst(person,"name");
			this.Email = Utils.getFirst(person,"email");
			this.Id = Utils.getFirst(person,"id");
			this.Passwd = Utils.getFirst(person,"passwd");
			this.Institute = Utils.getFirst(person,"institute");			
			this.Department = Utils.getFirst(person,"department");
			Node colors = el.getChildNodes().item(1);
			String color = Utils.getFirst(colors,"goodlink");
			this.GoodLinkColor = getColor(Utils.getFirst(colors,"goodlink"));	
			this.BadLinkColor = getColor(Utils.getFirst(colors,"badlink"));
			this.NeutralLinkColor = getColor(Utils.getFirst(colors,"neutrallink"));
			this.ExternalLinkColor = getColor(Utils.getFirst(colors,"externallink"));
			this.ExternalVisitedColor = getColor(Utils.getFirst(colors,"externalvisitedlink"));
			this.ActiveLinkColor = getColor(Utils.getFirst(colors,"activelink"));

			this.Knowledge = new Hashtable();
			Node knowledge = el.getChildNodes().item(2);
			NodeList knowledgeitems = knowledge.getChildNodes();
			for (int i=0; i<knowledgeitems.getLength(); i++) {
				Node item = knowledgeitems.item(i);
				String name = Utils.getFirst(item,"knowledgename");
				String perc = Utils.getFirst(item,"knowledgeperc");
				this.Knowledge.put(name,perc);
				// System.out.println(name+" -- "+perc);
			}
			this.VisitedList = new Hashtable();
			Node visited = el.getChildNodes().item(3);
			NodeList visitedlist = visited.getChildNodes();
			for (int i=0; i<visitedlist.getLength(); i++) {
				Node item = visitedlist.item(i);
				String name = Utils.getFirst(item,"visitedname");
				String perc = Utils.getFirst(item,"visitedperc");
				// System.out.println(name+" == "+perc);
				this.VisitedList.put(name,perc);
			}
		} catch (Exception e) {
			System.out.println("Error reading UserModel file: "+ModelName+"/model");
			Messages m = AHA.getMessages();
			m.Fatal("Error reading UserModel file: "+ModelName+"/model. Please notify the author about this error.");
		}}
		// read in logfile


		FileName logfilename = new FileName(ModelName, "log");
		File LF = new File(logfilename.toString());

		if (LF == null || !LF.exists())
			throw new AhaException("Logfile does not exist.");
		try {
			BufferedReader LogFile = new BufferedReader(new FileReader(LF), 8192);
			LogFileLines = new Vector();
			String line;
			while ((line = LogFile.readLine()) != null) {
				LogFileLines.addElement(line);
			}
			LogFile.close();
		} catch (Exception ee) {throw new AhaException("Error reading Logfile: "+LF.toString());}
	}

	public Color getColor(String color) {
		return  new Color(Integer.decode("0x"+color).intValue());
	}

	public Element makeEl(XmlDocument d,String name, String value) {
		Text t = d.createTextNode(value);
		Element el = d.createElement(name);
		el.appendChild(t);
		return el;
	}
  /**
   * This method rewrites the User Model file and appends to the log file.
   * The ModelName is supposed to be a unique object for this name,
   * because it is used for synchronization between threads.
   * The ModelName is supposed to be a unique object for this name,
   * because it is used for synchronization between threads.
   *
   * @exception	AhaException	If the user model cannot be saved or
   * the log file cannot be appended to.
   */

	public synchronized void saveUserModel(String ModelName) throws AhaException{

		FileName filename = new FileName(ModelName, "model");
		println("Saving Usermodel: "+filename.toString());
		synchronized(ModelName) {

			XmlDocument d = new XmlDocument();
			Element name = makeEl(d,"name",this.Name);
			Element email = makeEl(d,"email",this.Email);
			Element id = makeEl(d,"id",this.Id);
			Element passwd = makeEl(d,"passwd",this.Passwd);
			Element institute = makeEl(d,"institute", this.Institute);
			Element department = makeEl(d,"department", this.Department);
			Element person = d.createElement("person");
			person.appendChild(name);
			person.appendChild(email);
			person.appendChild(id);
			person.appendChild(passwd);
			person.appendChild(institute);
			person.appendChild(department);

			Element goodlinkcolor = makeEl(d,"goodlink", printGoodLinkColor());
			Element badlinkcolor = makeEl(d,"badlink", printBadLinkColor());
			Element neutrallinkcolor = makeEl(d,"neutrallink", printNeutralLinkColor());
			Element externallinkcolor = makeEl(d,"externallink", printExternalLinkColor());
			Element externalvisitedcolor = makeEl(d,"externalvisitedlink", printExternalVisitedColor());
			Element activelinkcolor = makeEl(d,"activelink", printActiveLinkColor());			

			Element colors = d.createElement("colors");
			colors.appendChild(goodlinkcolor);
			colors.appendChild(badlinkcolor);
			colors.appendChild(neutrallinkcolor);
			colors.appendChild(externallinkcolor);
			colors.appendChild(externalvisitedcolor);
			colors.appendChild(activelinkcolor);

			Element knowledgelist = d.createElement("knowledgelist");
			Enumeration e = Knowledge.keys();
			while (e.hasMoreElements()) {
				String kname = (String)e.nextElement();
				String kvalue = (String)Knowledge.get(kname);
				Element knowledgename = makeEl(d,"knowledgename",kname);
				Element knowledgeperc = makeEl(d,"knowledgeperc",kvalue);
				Element knowledgeitem = d.createElement("knowledgeitem");
				knowledgeitem.appendChild(knowledgename);
				knowledgeitem.appendChild(knowledgeperc);
				knowledgelist.appendChild(knowledgeitem);
			}

			Element visitedlist = d.createElement("visitedlist");
			e = VisitedList.keys();
			while (e.hasMoreElements()) {
				String vname = (String)e.nextElement();
				String vvalue = (String)VisitedList.get(vname);
				println(vname+" --- "+vvalue);
				Element visitedname = makeEl(d,"visitedname",vname);
				Element visitedperc = makeEl(d,"visitedperc",vvalue);
				Element visitedlistitem = d.createElement("visitedlistitem");
				visitedlistitem.appendChild(visitedname);
				visitedlistitem.appendChild(visitedperc);
				visitedlist.appendChild(visitedlistitem);
			}

			Element usermodel = d.createElement("usermodel");
			usermodel.appendChild(person);
			usermodel.appendChild(colors);
			usermodel.appendChild(knowledgelist);
			usermodel.appendChild(visitedlist);
			d.appendChild(usermodel);
			d.setSystemId("usermodel");
			d.setDoctype(null,"../../../WWW/AHAstandard/usermodel.dtd",null);
			try {
				PrintWriter pw = new PrintWriter(new FileWriter(filename.toString()));
				d.write(pw);
				pw.close();
			} catch (Exception ee) {System.out.println("Error writing xml document: "+filename);
				ee.printStackTrace();
			}

			// now update the logfile
			FileName logfilename = new FileName(ModelName, "log");
			try {
				RandomAccessFile LogFile = new RandomAccessFile(logfilename.toString(), "rw");
				if (LogFile == null)
					throw new AhaException("Cannot update the logfile.");
				int limit = LogFileLines.size();
				int i = limit-1;
				if (i>=0) {
					// when registering we can't write a log entry yet
					LogFile.seek(LogFile.length());
		  			LogFile.write(((String)LogFileLines.elementAt(i)+"\n").getBytes(), 0, ((String)LogFileLines.elementAt(i)).length()+1);
				}
				LogFile.close();
			} catch (IOException ioe) {
				System.out.println("Error writing logfile: "+logfilename.toString());
				throw new AhaException("An I/O error occurred while saving logfile, "+ioe.toString());
			}
		}// end synchronize
	}


  /**
   * Return the internal representation of the user's knowledge.
   */
  public Hashtable getKnowledge() {
    return Knowledge;
  }

  /**
   * Replace the user's knowledge by the argument of the method call.
   */
  public void setKnowledge(Hashtable t) {
    Knowledge = t;
  }


	/**
	 * Return the internal representation of the pages with their visited status
	 */
	public Hashtable getVisitedList() {
		return VisitedList;
	}

	/**
	 * Replace the user's pages with visited status
	 */
	public void setVisitedList(Hashtable hs) {
		this.VisitedList = hs;
	}

  /**
   * Return the lines of the user's log file as a vector.
   */
  public Vector getLogFileLines() {
    return LogFileLines;
  }

  /**
   * Replace the user's vector representation of the log file by the argument
   * of the method call.
   */
  public void setLogFileLines(Vector v) {
    LogFileLines = v;
  }

  /**
   * Return a String representation of a Color. The String has the hexadecimal
   * pattern required for color representations in HTML.
   */
  public static String printColor(Color c) {
    int i = c.getRed();
    String r,g,b;
    if (i<10)
      r = "0" + Integer.toHexString(c.getRed());
    else
      r = Integer.toHexString(c.getRed());
    i = c.getGreen();
    if (i<10)
      g = "0" + Integer.toHexString(c.getGreen());
    else
      g = Integer.toHexString(c.getGreen());
    i = c.getBlue();
    if (i<10)
      b = "0" + Integer.toHexString(c.getBlue());
    else
      b = Integer.toHexString(c.getBlue());
    return r + g + b;
  }

  /**
   * Return the color for desired links.
   */
  public Color getGoodLinkColor() {
    return GoodLinkColor;
  }

  /**
   * Return the hexadecimal representation of the color for desired links.
   */
  public String printGoodLinkColor() {
    return printColor(GoodLinkColor);
  }

  /**
   * Set the color for desired links.
   */
  public void setGoodLinkColor(Color c) {
    GoodLinkColor = c;
  }
  
  /**
   * Return the color for undesired links.
   */
  public Color getBadLinkColor() {
    return BadLinkColor;
  }

  /**
   * Return the hexadecimal representation of the color for undesired links.
   */
  public String printBadLinkColor() {
    return printColor(BadLinkColor);
  }

  /**
   * Set the color for undesired links.
   */
  public void setBadLinkColor(Color c) {
    BadLinkColor = c;
  }
  
  /**
   * Return the color for active links (being clicked on).
   */
  public Color getActiveLinkColor() {
    return ActiveLinkColor;
  }

  /**
   * Return the hexadecimal representation of the color for active
   * links (being clicked on).
   */
  public String printActiveLinkColor() {
    return printColor(ActiveLinkColor);
  }

  /**
   * Set the color for active links (being clicked on).
   */
  public void setActiveLinkColor(Color c) {
    ActiveLinkColor = c;
  }
  
  /**
   * Return the color for uninteresting (previously visited) links.
   */
  public Color getNeutralLinkColor() {
    return NeutralLinkColor;
  }

  /**
   * Return the hexadecimal representation of the color for uninteresting
   * (previously visited) links.
   */
  public String printNeutralLinkColor() {
    return printColor(NeutralLinkColor);
  }

  /**
   * Set the color for uninteresting (previously visited) links.
   */
  public void setNeutralLinkColor(Color c) {
    NeutralLinkColor = c;
  }
  
  /**
   * Return the color for external unvisited links.
   */
  public Color getExternalLinkColor() {
    return ExternalLinkColor;
  }

  /**
   * Return the hexadecimal representation of the color for external
   * unvisited links.
   */
  public String printExternalLinkColor() {
    return printColor(ExternalLinkColor);
  }

  /**
   * Set the color for external unvisited links.
   */
  public void setExternalLinkColor(Color c) {
    ExternalLinkColor = c;
  }
  
  /**
   * Return the color for external visited links.
   */
  public Color getExternalVisitedColor() {
    return ExternalVisitedColor;
  }

  /**
   * Return the hexadecimal representation of the color for external
   * visited links.
   */
  public String printExternalVisitedColor() {
    return printColor(ExternalVisitedColor);
  }

  /**
   * Set the color for external visited links.
   */
  public void setExternalVisitedColor(Color c) {
    ExternalVisitedColor = c;
  }
  
  /**
   * Return the user's name.
   */
  public String getName() {
    return Name;
  }

  /**
   * Set the user's name.
   */
  public void setName(String s) {
    Name = s;
  }

  /**
   * Return the user's email address.
   */
  public String getEmail() {
    return Email;
  }

  /**
   * Set the user's email address.
   */
  public void setEmail(String s) {
    Email = s;
  }

  /**
   * Return the user's id.
   */
  public String getId() {
    return Id;
  }

  /**
   * Set the user's id.
   */
  public void setId(String s) {
    Id = s;
  }

  /**
   * Return the user's password.
   */
  public String getPasswd() {
    return Passwd;
  }

  /**
   * Set the user's password.
   */
  public void setPasswd(String s) {
    Passwd = s;
  }

  /**
   * Return the user's institute.
   */
  public String getInstitute() {
    return Institute;
  }

  /**
   * Set the user's institute.
   */
  public void setInstitute(String s) {
    Institute = s;
  }

  /**
   * Return the user's department.
   */
  public String getDepartment() {
    return Department;
  }

  /**
   * Set the user's department.
   */
  public void setDepartment(String s) {
    Department = s;
  }

  /**
   * Return the number of read pages.
   */
  public int getNbRead() {
    return NbRead;
  }

  /**
   * Set the number of read pages.
   */
  public void setNbRead(int n) {
    NbRead = n;
  }

  /**
   * Return the number of pages to read.
   */
	public int getNbPages() {
		if (NbPages == -1) recount();
		return NbPages;
	}

	public void recount() {
		NbNotVisited = 0;
		NbBadVisited = 0;
		NbGoodVisited = 0;
		NbPages = 0;
		Enumeration e = this.getVisitedList().keys();
		while (e.hasMoreElements()) {
			String key = (String)e.nextElement();
			String value = (String)(this.getVisitedList().get(key));
			value = value.trim();
			NbPages++;
			if (value.equals("0")) {
				NbNotVisited++;
			} else if (value.equals("100")) {
				NbGoodVisited++;
			} else NbBadVisited++; 
		}		
		println("not = "+NbNotVisited);
		println("bad = "+NbBadVisited);
		println("good = "+NbGoodVisited);
	}

  /**
   * Set the number of pages to read.
   */
  public void setNbPages(int n) {
    NbPages = n;
  }

	public int getNbNotVisited() {
		if (NbNotVisited == -1) recount();
		return NbNotVisited;
	}

	public int getNbBadVisited() {
		if (NbBadVisited == -1) recount();
		return NbBadVisited;
	}

	public int getNbGoodVisited() {
		if (NbGoodVisited == -1) recount();
		return NbGoodVisited;
	}		

	public boolean hasNotVisit(String name) {
		boolean b = false;
		if (VisitedList.containsKey(name)) {
			int p = Integer.parseInt((String)VisitedList.get(name));
			if (p==0) b = true;
		} else {
			b = true;
		}
		return b;
	}

	public boolean hasBadVisit(String name) {
		boolean b = false;
		if (VisitedList.containsKey(name)) {
			int p = Integer.parseInt((String)VisitedList.get(name));
			if (p>0 && p<100) b = true;
		}
		return b;
	}
	public boolean hasGoodVisit(String name) {
		boolean b = false;
		if (VisitedList.containsKey(name)) {
			int p = Integer.parseInt((String)VisitedList.get(name));
			if (p==100) b = true;
		}
		return b;
	}
}
