package AHA;

import XmlUtils.*;
import Expression.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import org.w3c.dom.*;


	/**
	 * XmlFilter is used for parsing XML documents, and it returns a HTML representation of the data
	 */

public class XmlFilter {
	private boolean debug = true;
	protected AHA AHA;
	protected PrintWriter out;
	protected Hashtable Knowledge;
	protected String FullDocName;
	protected String DocName;
	public final String ifstring = "if";
	public final String headerstring = "header";
	public final String footerstring = "footer";
	public final String printattr = "print";
	public final String fileattr = "file";
	public final String htmlstring = "htmlcode";
	public final String anchorstring = "A";

	public Node footerNode;


	/**
	 * The default constructor is forbidden
	 * @exception AhaException Always thrown because this constructor is not valid
	 */
	public XmlFilter() throws AhaException {
		throw new AhaException("Invalid call to default XmlFilter constructor.");
	}


	/** 
	 * This constructor is for base construction
	 * @param AHA the AHA context
	 */
	public XmlFilter(AHA AHA) {
		this.AHA = AHA;
		this.out = AHA.getWriter();
		this.Knowledge = AHA.getUserModel().getKnowledge();
	}
	
	/** 
	 * This constructor may be used for outside calls
	 * @param AHA pass the general AHA object
	 * @param FullDocName pass the exact url for the datadocument
	 * @param DocName the short name of the document
	 */
	public XmlFilter(AHA AHA, String FullDocName, String DocName){
		this(AHA);
		this.FullDocName = FullDocName;
		this.DocName = DocName;
		println("XmlFilter: "+FullDocName);
	}

	public void println(Object s) { if (debug) System.out.println(s.toString());}
	public void print(Object s) { if (debug) System.out.print(s.toString());}

	/**
	 * Used for adjusting the color of the anchor
	 * @param h name of page
	 */
	public String hasVisited(String h) {
		String href = h;
		if (h.indexOf(".xml")>-1) href = h.substring(0,h.indexOf(".xml"));
		UserModel UserModel = AHA.getUserModel();
		if (UserModel.hasNotVisit(href)) return "Good";
		if (UserModel.hasBadVisit(href)) return "Good";
		if (UserModel.hasGoodVisit(href)) return "Neutral";
		return "ERROR";
	}

	/** 
	 * Used for adjusting the color of the anchor
	 * @param href name of page
	 */
	public String isGood(String href) {
		RequirementList cr = AHA.getRequirementList();
		String h = href.substring(0,href.indexOf(".xml"));
		if (cr.containsKey(h)) {
			Expr expr = (Expr)cr.get(h);
			if (expr.eval(Knowledge)) {
				return hasVisited(href);
			} else return "Bad";
		}
		else {
			return hasVisited(href);
		}
	}
		
	/**
	 * given an anchorNode change the 'class' attribute according
	 * to the user's knowledge
	 * @param anchorNode the anchor object
	 */
	public String FilterLink(Node anchorNode) throws AhaException{
		
		Element anchor = (Element)anchorNode;
		println("Filter anchor: "+anchor.toString());
	
		String href = anchor.getAttribute("href");

		if (href.indexOf(":")>-1) {
			println("External: "+anchor.toString());
		}

		String name = anchor.getAttribute("name");
		String aclass = anchor.getAttribute("class");
		String target = anchor.getAttribute("target");
	//	if (anchor.hasAttribute("class")) aclass = anchor.getAttribute("class");
		String other = anchor.getAttribute("other");
		if (aclass == null || aclass.equals("")) {
			if (href.indexOf(":")==-1 && !href.startsWith("test___") && !href.startsWith("eval___")) {
				anchor.setAttribute("class",hasVisited(href));
			} else if (href.startsWith("test___")) {
				anchor.setAttribute("class","Good");
			}
		} else {	// class != null
			if (aclass.equalsIgnoreCase("conditional")) {
				if (href.indexOf(":") == -1) {
					anchor.setAttribute("class",isGood(href));
				} else {
					// print warning
					println("External link with CLASS??");
				}

			} else if (aclass.equalsIgnoreCase("unconditional")) {
				if (href.indexOf(":") == -1) {
					anchor.setAttribute("class",hasVisited(href));
				} else {
					// print warning
					println("External Link with CLASS?");
				}
			} else {
				throw new AhaException("Unknown anchor class. Please notify the author");
			}
		}
		int l = href.indexOf("DoTest");
		if (l > -1) {
			// this is a link to a test
			UserModel UserModel = AHA.getUserModel();
			String Id = UserModel.getId();
			String Course = AHA.getCourse();
			String begin = href.substring(0, l + 7);
			String eind = href.substring(l+7);
			href = begin+Course+"/"+Id+"/"+eind;
		}
		StringBuffer value = printChildNodes(anchor, true);
		println("value = "+value);
		String an = new String("<a ");
		if (!href.equals("")) an += "href=\""+href+"\" ";
		if (!name.equals("")) an += "name=\""+name+"\" ";
		if (anchor.getAttribute("class") != null) 
			an += "class=\""+anchor.getAttribute("class")+"\" ";
		//if (!aclass.equals("")) an += "class=\""+anchor.getAttribute("class")+"\" ";
		if (!target.equals("")) an += "target=\""+target+"\" ";
		if (!other.equals("")) an += "other=\""+other+"\"";
		an += ">"+value+"</a>";
		println(an);
		return an;
	}

	/**
	 * test an expression to the users knowledge
	 * @param s the expression to be checked
	 */
	public boolean teststr(String s) {
		try {
			Expr expr = new Expr().parseTerm(s);
			boolean b = expr.eval(Knowledge);
			println("Expr: "+s+"   ("+expr.toString()+") ==  "+b);
			return b;
		} catch (ExprException ee) {
			System.out.println("XmlFilter Error in expression: "+s+" (not parseable).");
			return true;	// default???? CHANGE
		}
	}	

	/**
	 * Node is an if-tag. The expression is given as an attribute "expr".
	 * @param n The if tag
	 */
	public boolean teststr(Node n) {
		Element e = (Element)n;
		return teststr(e.getAttribute("expr"));
	}

	/**
	 * Is this node an if-tag?
	 */
	public boolean isIfStatement(Node node) {
		return node.getNodeName().equals(ifstring);
	}

	/** 
	 * Is this node a header tag?
	 */
	public boolean isHeaderStatement(Node node) {
		return node.getNodeName().equals(headerstring);
	}

	/** 
	 * Is this node a footer tag?
	 */
	public boolean isFooterStatement(Node node) {
		return node.getNodeName().equals(footerstring);
	}

	/**
	 * Is this node a htmltag?
	 */
	public boolean isHtmlCode(Node node) {
		return node.getNodeName().equals(htmlstring);
	}

	/**
	 * Is this node an anchor tag?
	 */
	public boolean isAnchorStatement(Node node) {
		return node.getNodeName().toUpperCase().equals(anchorstring);
	}

	/**
	 * Is this node a block tag?
	 */
	public boolean isBlockNode(Node node) {
		return node.getNodeName().equals("block"); //blockstring);
	}

	/**
	 * Maybe this is only used for testing
	 */
	public StringBuffer printAttr(Node node1) {
		StringBuffer sb = new StringBuffer();
		if (node1 instanceof Element) {
			Element node = (Element)node1;
			if (isIfStatement(node)) {
				String expr = node.getAttribute("expr");
				sb.append("IF "+expr+" = "+teststr(expr));
			}
		}
		//println(sb);
		return new StringBuffer();
	}

	/**
	 * This method gets an if-tag and depending on the expression returns a block
	 * if if-expr=true --> first block	(item(0))
	 * if if-expr=false --> second block if any	(item((1))
	 */
	public StringBuffer makeIfChoice(Node node) throws AhaException {

		StringBuffer sb = new StringBuffer();
		NodeList nodelist = node.getChildNodes();
		boolean b = teststr(node);
		if (b) {
			sb.append(printNode(nodelist.item(0),true));
		}
		if (!b && nodelist.getLength()==2) {
			sb.append(printNode(nodelist.item(1),true));
		}
		return sb;
	}
		

	/**
	 * This method takes care of writing headers and footers,
	 * which consists of calling to the Messages object
	 */
	public StringBuffer doHeaderFooter(Node node) throws AhaException{
		StringBuffer sb = new StringBuffer();
		boolean print = ((Element)node).getAttribute(printattr).equals("yes");
		String filename ="";
		filename = ((Element)node).getAttribute(fileattr);
		println("filename = "+filename);
		if (filename.equals("")) filename = null;
		if (isHeaderStatement(node)) {
			if (print) 
				sb.append(AHA.getMessages().PrintBodyTag());
			if (filename !=null) {
				HeaderFooterFilter hff = new HeaderFooterFilter(AHA, filename, DocName);
				sb.append(hff.makeHTML());
			}
		}
		if (isFooterStatement(node)) {
			if (filename != null) {
				HeaderFooterFilter hff = new HeaderFooterFilter(AHA, filename, DocName);
				sb.append(hff.makeHTML());
			}
			if (print) 
				sb.append(AHA.getMessages().PrintEndBodyTag());
		}
		return sb;
	}	

	/**
	 * This returns a string containing an anchor
	 */
	public String filterAnchorNode(Node node) throws AhaException{
		// here the anchor (<a>) tag can be changed
		return FilterLink(node).toString();
	}


	/**
	 * Get the (html) representation of this node
	 * @param b if b then generate something, else generate nothing
	 */	
	public StringBuffer printNode(Node node, boolean b) throws AhaException {
		StringBuffer sb = new StringBuffer();
		if (b) {
			String name = node.getNodeName();
			String value = node.getNodeValue();
			//println("name="+name+" value="+value);
			if (isIfStatement(node)){ 
				sb.append(printAttr(node)+"\n");
				sb.append(makeIfChoice(node));
			}
			else if (isHeaderStatement(node) || isFooterStatement(node)) {
					sb.append(doHeaderFooter(node));
			} else  if (isAnchorStatement(node)) {
				//System.out.println("Anchor: "+node.toString());
				sb.append(filterAnchorNode(node).toString());
			}
			else if (isBlockNode(node)) {
				if (node.hasChildNodes()) {
					sb.append(printChildNodes(node,true));
				}
			} else {	// htmlcode
				//System.out.println(node.toString());
				if (node instanceof Text) {
					//System.out.print("Name:"+name+"="+node.toString().trim());
				}
				sb.append(node.toString());
			}
		}
		return sb;
	}

	/**
	 * Get the (html) representation of the childs of this node
	 * @param b pass this boolean to printNode
	 */
	public StringBuffer printChildNodes(Node nodec, boolean b) throws AhaException {
		StringBuffer sb = new StringBuffer();
		NodeList nodelist = nodec.getChildNodes();
		for (int i=0; i<nodelist.getLength(); i++) {
			Node node = nodelist.item(i);
			sb.append(printNode(node,b));
		}
		return sb;
	}

	/**
	 * prints out the html to AHA.getWriter
	 */
	public void run() throws AhaException{
		try {
			XmlReader xml = new XmlReader(FullDocName);

			Element el = xml.openFile();
			StringBuffer buffer = printChildNodes(el,true);
			// don't forget to make html code from buffer
			// buffer.append(doHeaderFooter(footerNode));
			out.println(buffer);
			out.flush();

		} catch (Exception e) {
			System.out.println("Error reading file: "+FullDocName);
			e.printStackTrace();
			
			Messages m = AHA.getMessages();
			m.Fatal("Error reading file: "+FullDocName+". Please notify the author.");
			throw new AhaException("Error: "+e.toString());
		}
	}

}	