import javax.swing.table.*;
import java.util.*;
import AHA.*;
	class ConceptTableDef extends AbstractTableModel {
      protected static int NUM_COLUMNS = 4;
      protected static int START_NUM_ROWS = 1;
      protected int nextEmptyRow = 0;
      protected int numRows = 0;
      protected int sortCol = 0;

      static final public String PConceptName = "Concept";
      static final public String changeableName = "Changeable";
      static final public String KConceptName = "Composite";
      static final public String PercentageName = "Percentage";

	 	  private Vector data = null;
		  private int[] disabledRows;

      public ConceptTableDef(int[] disabled) {
        data=new Vector();
 			  disabledRows = disabled;
			}

      public void setsortCol(int sort) {
        sortCol=sort;
      }

      public int getsortCol() {
        return sortCol;
      }

      public void setDisabledRows(int[] disabled) {
        disabledRows=disabled;
      }

      public int[] getDisadbledRows() {
        return disabledRows;
      }

		  public boolean isCellEditable(int row, int col) {
        return false;
			}

      //XXX Should this really be synchronized?
      public synchronized int getColumnCount() {
          return NUM_COLUMNS;
      }

      public synchronized int getRowCount() {
          if (numRows < START_NUM_ROWS) {
              return START_NUM_ROWS;
          } else {
              return numRows;
          }
      }

      public synchronized Object getValueAt(int row, int column) {
      	try {
          ConceptRecord c = (ConceptRecord)data.elementAt(row);
          switch (column) {
            case 0:
              return c.concept;
            case 1:
              return c.changeable;
            case 2:
              return c.composite;
            case 3:
              return c.percentage;
          }
      	} catch (Exception e) {
      	}
	      return "";
      }

      public void Sort() {
          Collections.sort (data, new ConceptComparator());
          int i=1;
          int j=0;
          if (sortCol==0) {
            while (i<data.size ()) {
              if (((ConceptRecord)data.elementAt(i)).concept.equals (((ConceptRecord)data.elementAt(i-1)).concept)) {
                disabledRows[j]=i;
                j++;
              }
              i++;
            }
          } else {
            while (i<data.size ()) {
              if (((ConceptRecord)data.elementAt(i)).composite.equals (((ConceptRecord)data.elementAt(i-1)).composite)) {
                disabledRows[j]=i;
                j++;
              }
              i++;
            }
          }
          for (int n =j;n<disabledRows.length;n++) {
            disabledRows[n]=-1;
          }
          this.setDisabledRows (disabledRows);
          fireTableDataChanged();
      }

      public synchronized void addConcept(ConceptRecord cr) {
          if (numRows <= nextEmptyRow) {
            //add a row
            numRows++;
          }
    	    data.addElement (cr);
          fireTableRowsInserted(nextEmptyRow, nextEmptyRow);
          nextEmptyRow++;
      }

      public synchronized void addConceptRecord(ConceptRecord cr) {
          if (numRows <= nextEmptyRow) {
            //add a row
            numRows++;
          }
    	    data.addElement(cr);
          fireTableRowsInserted(nextEmptyRow, nextEmptyRow);
          nextEmptyRow++;
      }

      public synchronized void changeConcept(Object oldKey, Object newKey, boolean changeable) {
          String ID = oldKey.toString (); //find the ID
          int index = -1;
          int i = 0;
          ConceptRecord c;
          while (i < nextEmptyRow) {
              c = (ConceptRecord)data.elementAt(i);
              if (c.concept.equals (ID)) {
                  c.concept=newKey.toString ();
                  c.changeable = new Boolean(changeable);
                  data.setElementAt (c,i);
                  fireTableRowsUpdated(i,i);
              }
              i++;
          }
      }

      public synchronized void changeConceptRecord(ConceptRecord oldcr, ConceptRecord newcr) {
          String ID=oldcr.concept;
          String Concept=oldcr.composite;
          int i = 0;
          ConceptRecord c;
          while (i < nextEmptyRow) {
              c = (ConceptRecord)data.elementAt(i);
              if (c.concept.equals (ID) && c.composite.equals (Concept)) {
                  data.setElementAt (newcr,i);
                  fireTableRowsUpdated(0,i);
                  i=nextEmptyRow;
              }
              i++;
          }
      }

      public synchronized void deleteConcept(Object key) {
        String ID = key.toString (); //find the ID
        int index = -1;
        int i = 0;
        ConceptRecord c;
        while (i < nextEmptyRow) {
          c = (ConceptRecord)data.elementAt(i);
          if (c.concept.equals (ID)) {
            data.removeElementAt (i);
            nextEmptyRow--;
            numRows--;
            fireTableRowsDeleted(i,i);
          } else {
            i++;
          }
        }
      }

      public synchronized void deleteConceptRecord(String name, String Concept) {
        int index = -1;
        int i = 0;
        ConceptRecord c;
        while (i < nextEmptyRow) {
          c = (ConceptRecord)data.elementAt(i);
          if (c.concept.equals (name) && c.composite.equals (Concept)) {
            data.removeElementAt (i);
            nextEmptyRow--;
            numRows--;
            fireTableRowsDeleted(i,i);
          } else {
            i++;
          }
        }
      }

      public synchronized void clear() {
      	int oldNumRows = numRows;

        numRows = START_NUM_ROWS;
    	  data.removeAllElements();
        nextEmptyRow = 0;

        if (oldNumRows > START_NUM_ROWS) {
	        fireTableRowsDeleted(START_NUM_ROWS, oldNumRows - 1);
      	}
    	  fireTableRowsUpdated(0, START_NUM_ROWS - 1);
      }

		  public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        data.addElement (aValue);
      }

		  public boolean isRowDisabled(int row) {
			  for(int i = 0; i < disabledRows.length; i++) {
          if(disabledRows[i] == row) { return true; }
        }
        return false;
      }

      public String getColumnName(int column) {
      	switch (column) {
	        case 0:
            if (this.sortCol == 0) {
      	      return PConceptName + " \u2193";
            } else {
              return PConceptName;
            }
    	    case 1:
	          return changeableName;
      	  case 2:
            if (this.sortCol == 2) {
      	      return KConceptName + " \u2193";
            } else {
              return KConceptName;
            }
      	  case 3:
	          return PercentageName;
      	}
      	return "";
      }

      class ConceptComparator implements Comparator {
        public ConceptComparator() {
        }

        public int compare(Object o1, Object o2) {
          if (!(o1 instanceof ConceptRecord) || !(o2 instanceof ConceptRecord)) return 0;
          ConceptRecord cr1 = (ConceptRecord)o1;
          ConceptRecord cr2 = (ConceptRecord)o2;
          int result = 0;
          switch (sortCol) {
            case 0: //Page
              if (cr1.concept.compareToIgnoreCase(cr2.concept)==0) {
                result=cr1.composite.compareToIgnoreCase(cr2.composite);
              } else {
                result=cr1.concept.compareToIgnoreCase(cr2.concept);
              }
              break;
            case 2: //Concept
              if (cr1.composite.compareToIgnoreCase(cr2.composite)==0) {
                result=cr1.concept.compareToIgnoreCase (cr2.concept);
              } else {
                result=cr1.composite.compareToIgnoreCase(cr2.composite);
              }
              break;
          }
          return result;
        }
      }
  }

