package Expression;
import java.util.Hashtable;

public class Atom extends Expr{
	String name;

	// percentage is stored as a String, but converted to int when necessary.
	String percentage;

	// The comparator used in this Atom
	Comparator comparator;


	public Atom() {
	}


	/** 
	 * Atom constructor
	 * Parameters:
	 * @param name Name for this Atom
	 * @param percentage Percentage for this Atom currently present
	 */
	public Atom(String name, String percentage) {
		this.name = name;
		this.percentage = percentage;
	}

	/**
	 * Atom constructor/parser
	 * Parameters:
	 * @param s String to be parsed. String or StringCompPercentage.
	 */
	public Atom(String s) throws ExprException {
		Hashtable comp = new Hashtable();
		comp.put(":",":"); comp.put("=","=");
		comp.put(">",">"); comp.put("<","<");
		s = removeAllSpaces(s);	
		if (s.indexOf(":") == -1 && s.indexOf(">") == -1 && s.indexOf("<") == -1 && s.indexOf("=") == -1) {
			this.name = s;
			this.percentage = "100";	// MAX
			this.comparator = new Comparator(">=");
		} else { // s contains :, >, < or =
			int i = 0;
			while (i<s.length() && !comp.containsKey(new String().valueOf(s.charAt(i)))) {
				i++;
			}
			if (i>=s.length())
				throw new ExprException("Could not find comparator.");
			int j = s.length()-1;
			while (j>-1 && !comp.containsKey(new String().valueOf(s.charAt(j)))) {
				j--;
			}
			if (j==-1)
				throw new ExprException("Could not find comparator.");
			// s[0..i) = name, s(j..) = percentage
			name = s.substring(0,i);
			percentage = s.substring(j+1);
			comparator = new Comparator(s.substring(i,j+1));
		}
	}


	/**
	 * Remove all spaces from a String, used to make parsing easier.
	 */
	private String removeAllSpaces(String s) {
		s = s.trim();
		int p;
		while ((p = s.indexOf(" ")) > -1) {
			String begin = s.substring(0,p).trim();
			String eind = s.substring(p).trim();
			s = begin+eind;
		}
		return s;
	}
	
	/**
	 * @return the Conceptname field
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the Concept percentage field
	 */
	public String getPercentage() {
		return percentage;
	}

	/**
	 * @return the Comparator field
	 */
	public Comparator getComparator() {
		return comparator;
	}

	/**
	 * @return String representation of this Atom
	 */
	public String toString() {
		return getName()+getComparator().toString()+getPercentage();
	}

	// why is this needed? Why is it true??
	public boolean evaluate() {
		return true;
	}

	/**
	 * @param hs the Knowledge the user has
	 * @return has the user enough Knowlegde for this Atom
	 */
	public boolean eval(Hashtable hs) throws ExprException {
		// true is true
		if (this.name.equals("true")) return true;

		// translating String to int may throw an exception
		try {
			boolean b=false;
			if (hs.containsKey(this.name)) {
				String perc = (String)hs.get(this.name);
				int kennis;
				if (perc.equals(this.name)) {
					kennis = 100;
				} else { kennis = Integer.parseInt(perc);
				}
				int vereist = Integer.parseInt(this.percentage);
				b = comparator.compare(kennis, vereist);
			} else {
				int vereist = Integer.parseInt(this.percentage);
				b = comparator.compare(0, vereist);
			}
			return b;
		} catch (NumberFormatException nfe) {
			throw new ExprException("Only integer values are allowed.");
		}
	}		

	/**
	 * Debug test for this class.
	 */
	public static void main(String[] args) {
		try {
			Atom b = new Atom("bier");
			System.out.println("--> "+b.toString());
			Atom a = new Atom(" a  > =100 ");
			System.out.println("--> "+a.toString());
			Atom c = new Atom(" c2   = > 1 0 0 ");
			System.out.println("--> "+c.toString());
		} catch (ExprException e) {
			System.out.println(e.toString());
		}
	}
}
