import javax.swing.table.*;
import java.util.*;
import AHA.*;
import Expression.Expr;
	class RequirementTableDef extends AbstractTableModel {
      protected static int NUM_COLUMNS = 2;
      protected static int START_NUM_ROWS = 1;
      protected int nextEmptyRow = 0;
      protected int numRows = 0;
      protected int sortCol = 0;

      static final public String PConceptName = "Concept";
      static final public String KConceptName = "Composite";


	 	  private Vector data = null;
		  private int[] disabledRows;

      public RequirementTableDef(int[] disabled) {
        data=new Vector();
 			  disabledRows = disabled;
			}

      public void setsortCol(int sort) {
        sortCol=sort;
      }

      public int getsortCol() {
        return sortCol;
      }

      public void setDisabledRows(int[] disabled) {
        disabledRows=disabled;
      }

      public int[] getDisadbledRows() {
        return disabledRows;
      }

		  public boolean isCellEditable(int row, int col) {
        return false;
			}

      //XXX Should this really be synchronized?
      public synchronized int getColumnCount() {
          return NUM_COLUMNS;
      }

      public synchronized int getRowCount() {
          if (numRows < START_NUM_ROWS) {
              return START_NUM_ROWS;
          } else {
              return numRows;
          }
      }

      public synchronized Object getValueAt(int row, int column) {
      	try {
          RequirementRecord rr = (RequirementRecord)data.elementAt(row);
          switch (column) {
            case 0:
              return rr.concept;
            case 1:
              return rr.composite;
          }
      	} catch (Exception e) {
      	}
	      return "";
      }

      public void Sort() {
          Collections.sort (data, new RequirementComparator());
          int i=1;
          int j=0;
          if (sortCol==0) {
            while (i<data.size ()) {
              if (((RequirementRecord)data.elementAt(i)).concept.equals (((RequirementRecord)data.elementAt(i-1)).concept)) {
                disabledRows[j]=i;
                j++;
              }
              i++;
            }
          } else {
            while (i<data.size ()) {
              if (((RequirementRecord)data.elementAt(i)).composite.equals (((RequirementRecord)data.elementAt(i-1)).composite)) {
                disabledRows[j]=i;
                j++;
              }
              i++;
            }
          }
          for (int n =j;n<disabledRows.length;n++) {
            disabledRows[n]=-1;
          }
          this.setDisabledRows (disabledRows);
          fireTableDataChanged();
      }

      public synchronized void addRequirementRule(RequirementRecord rr) {
          if (numRows <= nextEmptyRow) {
            //add a row
            numRows++;
          }
    	    data.addElement (rr);
          fireTableRowsInserted(nextEmptyRow, nextEmptyRow);
          nextEmptyRow++;
      }

      public synchronized void changeRequirementRule(Object oldKey, RequirementRecord newRR) {
          String ID = oldKey.toString (); //find the ID
          boolean found = false;
          int i = 0;
          RequirementRecord rr;
          while (i < nextEmptyRow && !found) {
              rr = (RequirementRecord)data.elementAt(i);
              if (rr.concept.equals (ID)) {
                  data.setElementAt (newRR,i);
                  fireTableRowsUpdated(i,i);
                  found=true;
              }
              i++;
          }
      }

      public synchronized void changeConcept(Object oldKey, Object newKey, boolean changeable) {
          String ID = oldKey.toString (); //find the ID
          int index = -1;
          int i = 0;
          ConceptRecord c;
          while (i < nextEmptyRow) {
              c = (ConceptRecord)data.elementAt(i);
              if (c.concept.equals (ID)) {
                  c.concept=newKey.toString ();
                  c.changeable = new Boolean(changeable);
                  data.setElementAt (c,i);
                  fireTableRowsUpdated(i,i);
              }
              i++;
          }
      }

      public synchronized void deleteRequirementRule(Object key) {
        String ID = key.toString (); //find the ID
        int i = 0;
        RequirementRecord rr;
        while (i < nextEmptyRow) {
          rr = (RequirementRecord)data.elementAt(i);
          if (rr.concept.equals (ID)) {
            data.removeElementAt (i);
            nextEmptyRow--;
            numRows--;
            fireTableRowsDeleted(i,i);
          } else {
            i++;
          }
        }
      }

      public synchronized void deleteConceptRecord(String name, String Concept) {
        int index = -1;
        int i = 0;
        ConceptRecord c;
        while (i < nextEmptyRow) {
          c = (ConceptRecord)data.elementAt(i);
          if (c.concept.equals (name) && c.composite.equals (Concept)) {
            data.removeElementAt (i);
            nextEmptyRow--;
            numRows--;
            fireTableRowsDeleted(i,i);
          } else {
            i++;
          }
        }
      }

      public synchronized void clear() {
      	int oldNumRows = numRows;

        numRows = START_NUM_ROWS;
    	  data.removeAllElements();
        nextEmptyRow = 0;

        if (oldNumRows > START_NUM_ROWS) {
	        fireTableRowsDeleted(START_NUM_ROWS, oldNumRows - 1);
      	}
    	  fireTableRowsUpdated(0, START_NUM_ROWS - 1);
      }

		  public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        data.addElement (aValue);
      }

		  public boolean isRowDisabled(int row) {
			  for(int i = 0; i < disabledRows.length; i++) {
          if(disabledRows[i] == row) { return true; }
        }
        return false;
      }

      public String getColumnName(int column) {
      	switch (column) {
	        case 0:
            if (this.sortCol == 0) {
      	      return PConceptName + " \u2193";
            } else {
              return PConceptName;
            }
    	    case 1:
            if (this.sortCol == 2) {
      	      return KConceptName + " \u2193";
            } else {
              return KConceptName;
            }
      	}
      	return "";
      }

      class RequirementComparator implements Comparator {
        public RequirementComparator() {
        }

        public int compare(Object o1, Object o2) {
          if (!(o1 instanceof RequirementRecord) || !(o2 instanceof RequirementRecord)) return 0;
          RequirementRecord rr1 = (RequirementRecord)o1;
          RequirementRecord rr2 = (RequirementRecord)o2;
          int result = 0;
          switch (sortCol) {
            case 0: //Concept
              if (rr1.concept.compareToIgnoreCase(rr2.concept)==0) {
                result=rr1.composite.toString ().compareToIgnoreCase(rr2.composite.toString ());
              } else {
                result=rr1.concept.compareToIgnoreCase(rr2.concept);
              }
              break;
            case 1: //Composite
              if (rr1.composite.toString ().compareToIgnoreCase(rr2.composite.toString ())==0) {
                result=rr1.concept.compareToIgnoreCase(rr2.concept);
              } else {
                result=rr1.composite.toString ().compareToIgnoreCase(rr2.composite.toString ());
              }
              break;
          }
          return result;
        }
      }
  }

