/**
 * @(#)AhaServlet.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import javax.servlet.*;
import javax.servlet.http.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Enumeration;
import java.util.Date;
import java.util.Calendar;
/**
 * The parent for (almost) every servlet in this package. CHANGE
 */
public class AhaServlet extends HttpServlet {
	protected PrintWriter out = null;
	protected AHAclass AHA = null;
	protected String loginid = null;
	protected UserModel UserModel = null;
	//private Messages Messages = null;
	protected String referer = null;	// from which url came the request?

	protected String Host=null;	// The name or IP number of the server.
	                	// This is needed to tell the Stop applet where the server is.
	protected int Port = -1;  	// The port number of the server. Also needed by Stop.
	protected String HtDocs;  	// The DocumentRoot of the WWW server. In theory it should
	                	// be possible to deduce this from some environment info
        	        	// (like PATH_INFO and PATH_TRANSLATED) but alas this
                		// doesn't appear to work consistently accross platforms
				// and servers, so we just initialize it at startup.
	protected String Course;		// The name of the directory of the AHA document (relative
		                // to the DocumentRoot).
	protected String Logs;		// Where the server keeps log files. AHA will put the
		                // logs in a subdirectory "Course" of this dir.
	protected String Title;      // The title of the AHA document. Is also used as the title
	                   // of each HTML page.
	protected String Background; // The name of the background image (in the "icons" dir).
	protected String Language;   // The language for messages. Currently only Dutch and
		                // English are supported.


	public void init(ServletConfig config) throws ServletException {
		super.init(config);
	}
	protected void service(	HttpServletRequest request, 
				HttpServletResponse response) {
	}



	/**
	 * This method will be called as initialization by each child servlet
	 * for every request. The AHA object will be derived from the session
	 * in the request. If the initialization fails (bad or no AHA object)
	 * then an exception (AhaException) will be thrown
	 * @exception AhaException - if there is an error with the AHA object
	 */
	protected void initService(HttpServletRequest request, HttpServletResponse response) throws AhaException{
		response.setContentType("text/html");
		printInfo();
		Timer t1 = new Timer();
		try {
			// this must use HttpUtils because otherwise the url is wrong
			String URI = javax.servlet.http.HttpUtils.getRequestURL(request).toString(); //.getRequestURI();


                        // The request's URI is needed to determine the identity of the
                        // the requested document (if present).
                        if (URI == null)
				throw new AhaException("Internal error. This request does not appear to provide a \"URI\" value. This indicates an error in the configuration of this document. Please notify the author that this error occurred.");
			int temp = URI.indexOf("/");
                        if (temp == -1)
				throw new AhaException("This request doesn't contain a name of a page. Please notify the author.");

			referer = javax.servlet.http.HttpUtils.getRequestURL(request).toString();

			if (referer.indexOf("/Get/") == -1) {
				// if it was not an access to a `normal' page
				// referer = null;
			}
			// set page-caching off
			setNoCache(request, response);

			response.setContentType("text/html");
			HttpSession session = request.getSession(false);
			if (session == null) 
				throw new AhaException("Sorry, there is no session.");

			try {
				loginid = new Derive(URI).getLoginid();
				Course = new Derive(URI).getCourse();
			} catch (Exception eee) {
				System.out.println("Name error: loginid or course is wrong");
				new Error("loginid or course is wrong");
			}

			// get the AHA object which is accessable via an AHASession
			ServletContext sc = this.getServletContext();
			AHASession ses = (AHASession)sc.getAttribute(loginid+"@"+Course);
			if (ses == null) 
				throw new AhaException("Wrong Course/login information for session.");

			if (ses.getSession() != session) 
				throw new AhaException("Wrong session.");
			AHA = ses.getAHA();
		

	synchronized(AHA) {
			if (AHA == null) 
				throw new AhaException("Error: AHA-parameters lost.");
			// Retrieve all important information from AHA
			UserModel = AHA.getUserModel();
			loginid = UserModel.getId();
			HtDocs = AHA.getHtDocs();
			Course = AHA.getCourse();
			Logs = AHA.getLogs();
			Title = AHA.getTitle();
			Background = AHA.getBackground();
			Language = AHA.getLanguageName(AHA.getLanguage());
			Host = AHA.getHost();
			Port = AHA.getPort();

			String missing = "";

			if (loginid == null) 
				missing += "loginid ";
			if (UserModel == null) 
				missing += "UserModel ";
			if (HtDocs == null) 
				missing += "HtDocs ";
			if (Course == null) 
				missing += "Course ";
			if (Logs == null) 
				missing += "Logs ";
			if (Title == null) 
				missing += "Title ";
			if (Background == null) 
				missing += "Background ";
			if (Language == null) 
				missing += "Language ";
			if (loginid == null || UserModel == null || HtDocs == null
			  || Course == null || Logs == null || Title == null 
			  || Background == null || Language == null) {
				throw new AhaException("Initial parameters missing: "+missing);
			}

			out = new PrintWriter(response.getWriter());
			AHA.setWriter(out);

			Messages Messages = new Messages(AHA);
			AHA.setMessages(Messages);
			Messages.initHeader();
			logAction(URI);
			log(URI);
	}
		} catch (Exception e) {
			System.out.println(getName() + ": " + e.toString());
			//e.printStackTrace();
			throw new AhaException(e.toString());
		}
		Timer t2 = new Timer();
	}			

	/**
	 * The method redirect will be called by child servlets if
	 * their call to super.initService(request, response) throws an error
	 * The user will get an error message on his screen
	 */
	public void redirect(HttpServletRequest request, HttpServletResponse response) {
		// Actually it is not a redirect, but it is called redirect 
		// for historical reasons
		try {
			response.setContentType("text/html");
			out = new PrintWriter(response.getWriter());
			out.println("<h1>AHA Error</h1>");
			out.println("Some important parameters are missing. Try to login to the system again.<p>");

			referer = javax.servlet.http.HttpUtils.getRequestURL(request).toString();
			String link = "/"+ new Derive(referer).getCourse();
			out.println("Possibly, this is a link to this Course loginpage:");
			out.println("<a href=\""+link+"\" target=\"_top\">"+link+"</a>");


			out.close();
		} catch (Exception ioe) {
			System.out.println("AhaServlet Exception: "+ getName()+" "+ioe.toString());
		}
	}


	/**
	 * Return the name of the servlet class.
	 * Inherited by childservlets.
	 * Used for tracking errors.
	 */
	public String getName() {
		return getClass().getName();
	}


	public void printInfo() {
		ServletContext sc = this.getServletContext();
		Enumeration e = sc.getAttributeNames();
		while (e.hasMoreElements()) {
			String name = (String)e.nextElement();
			Object value = sc.getAttribute(name);
		}
	}



	private void logAction(String URI) throws AhaException {
		String message = "";
		String action = ""; 
		String document = "";
		try {
			action = new Derive(URI).getServlet();
			document = new Derive(URI).getDoc();
		} catch (AhaException ahae) {
			System.out.println(ahae.toString());
		}
		if (action != null) message = message.concat(action+"/");
		if (document != null) message = message.concat(document);

		Date date = Calendar.getInstance().getTime();
		message = message.concat(" "+date.getTime()/1000);
		
		AHA.getUserModel().addLogFileLine(message, AHA.getUserModel().getId());
	}

	/**
	 * Method setNoCache is used for turning off page-caching by
	 * browsers, with respect to the HTTP version.
	 */
	private void setNoCache (HttpServletRequest request, HttpServletResponse response) {
		if (request.getProtocol().compareTo ("HTTP/1.0") == 0) {
			response.setHeader ("Pragma", "no-cache");
			response.setDateHeader ("Expires", 0);
		} else if (request.getProtocol().compareTo("HTTP/1.1") == 0) {
			response.setHeader ("Cache-Control", "no-cache");
			}

	}
}
