/*
 * @(#)Generate.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.util.StringTokenizer;
import java.util.Hashtable;

/**
 * The class Generate is used for generating knowledge for a user.
 * The GenerateList class provides the knowledge generation clauses.
 */

public class Generate{
	private int perc;
	private String name;
	private AHAclass AHA;
	private ConceptsList ConceptsList;

	// ceil takes care off rounding of doubles.
	// ceil = true for a "Generates_rest" of "Generates_full" and false otherwise
	// if ceil then round up, else down

	private boolean ceil;

        /**
	 * Generate "perc" knowledge for a page.
	 */
	public Generate(AHAclass AHA, String name, int perc,boolean ceil) throws AhaException{
		this.AHA = AHA;
		this.name = name;
		this.perc = perc;
		this.ceil = ceil;
		this.ConceptsList = AHA.getConceptsList();
			// lift yourself with perc points
		if (updateKnowledge())
			// lift children with perc percent but only if there was an update
		    updateChildren();
	}

	/**
	 * Update knowledge of this concept (name)
	 */
	public boolean updateKnowledge() {
		Hashtable Knowledge = AHA.getUserModel().getKnowledge();
		if (Knowledge.containsKey(this.name)) {
			String oldperc = (String)Knowledge.get(name);
			int oldp = Integer.parseInt(oldperc);
			int newp = oldp+perc;

			if (newp == oldp) // nothing changed
			    return false;
			// now clipping to the range [0..100]
			if (newp<0) newp = 0;
			    else
				if (newp>100) newp = 100;
			perc = newp - oldp; // this is the real update that was performed
			Knowledge.put(name,new Integer(newp).toString());
		} else {	// knowledge was zero
			Knowledge.put(name,new Integer(perc).toString());
		}
		return true;
	}

	/**
	 * Update knowledge of the children of this concept (name)
	 */
	public void updateChildren() throws AhaException{
		if (ConceptsList.containsKey(name)) {
			Concept c = (Concept)ConceptsList.get(name);
			
			String genlist = c.getGenerates();

			// now parse the concepts in genlist
			StringTokenizer st = new StringTokenizer(genlist);
			while (st.hasMoreTokens()) {
				String n = st.nextToken();
				int p = n.indexOf(":");
				boolean absolute = false;
				boolean negative = false;
				if (p>-1) {	// parse number en zo
					String nname = n.substring(0,p);
					String action = n.substring(p+1);
					int number = 0;;
					try {
						if (action.startsWith("+")) {
							number = Integer.parseInt(action.substring(1));
						} else if (action.startsWith("-")) { // number is negative
						        number = Integer.parseInt(action);
							negative = true;
						} else {	// number has no sign
							number = Integer.parseInt(action);
							absolute = true;
						}
					} catch (NumberFormatException nfe) {
						System.out.println("The number cannot be parsed: "+action);
					}
					if (absolute) { // do absolute update and stop without recursion
					        Hashtable Knowledge = AHA.getUserModel().getKnowledge();
					        Knowledge.put(nname,new Integer(number).toString());
					} else {
					        double doubleperc = perc;
					        double doublenumber = number;
						double doublep = (doubleperc / 100) * doublenumber;
						int liftperc = roundDouble(doublep);
						if (nname.equals(name) || negative) {
						    // self update or non-monotonic
						    Hashtable Knowledge = AHA.getUserModel().getKnowledge();
						    String oldperc = (String)Knowledge.get(nname);
						    int oldp = Integer.parseInt(oldperc);
						    int newp = oldp+liftperc;
						    if (newp<0) newp = 0;
						    else
							if (newp>100)newp=100;
						    Knowledge.put(nname,new Integer(newp).toString());
						} else {
						    Generate gchild = new Generate(AHA,nname,liftperc,this.ceil);
						}
					}
				} else {
					throw new AhaException("Internal Error: Generate knowledge fires unknown action");
				}
			}
		} else {
			// end of fire tree leaf
		}
	}

	/**
	 * roundDouble returns the ceiling of floor of a double, depending
	 * on the boolean "ceil" field.
	 */
	public int roundDouble(double d) {
	
		if (this.ceil) return (int)Math.ceil(d);
		else return (int)Math.floor(d);
	}
}
