/*
 * @(#)HTMLTest.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.util.Vector;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Enumeration;

public class HTMLTest {

  private StringBuffer XMLBuf; // Used to store the test in XML-format
	private String name;	// name of the file to write in the test
  private String title; // Title of the test  
  private String verbose; // verbose true/false
  private String number; // number of questions
  private int realnumber; // real number of questions, the empty ones left away
  private String show;   // number of questions to show
  private int maxanswers; // the maximum number of answers
  private String Indent="";
  Vector Questions;
  PrintWriter out;
  Vector keys; // used to save the keys of the CgiUtil in
  Vector values; // used to save the value of the CgiUtil in

  public HTMLTest(CgiUtil util, PrintWriter outt) 
  throws AhaException {

    // Create a new questions vector, to store the questions in
    XMLBuf=new StringBuffer(); // Used to store the test in XML-format	
    String key;
    String value;	
  
    out=outt;  // Save the output variabele for use with other methods

    keys=new Vector();
    values=new Vector();

    // Save all the keys with their corresponding values
    // this is done because reading an enumeration twice causes problems (?)

    Enumeration e=util.keys();
    while (e.hasMoreElements()) {
      key=(String)e.nextElement();
      value=(String)util.get(key);
      
	if (key.equals("name")) {
		setFileName(value);
	} else { 
		keys.addElement(key);
		values.addElement(value);
	}
    }  

    // get the various types of information from the form and put it in the buffer

    // first, check for the values of number, title, verbose and show	
    for (int i=0;i<keys.size();i++) {
   
      key=(String)keys.elementAt(i);
      value=(String)values.elementAt(i);

      if (key.equals("title"))
        title=value;
      else if (key.equals("show"))
        show=value;
      else if (key.equals("verbose"))
        verbose=value;
      else if (key.equals("number"))
        number=value;
      else if (key.equals("maxanswers"))
	maxanswers=Integer.valueOf(value).intValue();
    }


    //Now we will read all the question data and store it
    
    Questions=new Vector();
    int nq; // Number of the current question
    int na; // Number of the current answer

    realnumber=0;    
    // allocate memory for the questions + answers
    for (int j=0;j<Integer.valueOf(number).intValue();j++) 
      Questions.addElement(new Vraag(maxanswers));
   
    for (int i=0;i<keys.size();i++) {
      key=((String)keys.elementAt(i)).trim();
      value=((String)values.elementAt(i)).trim();
     
//      out.println("key: "+key+" value: "+value);
      // Are we reading the question?

      if (key.startsWith("question")) {
        // get the number of the question	
        nq=Integer.valueOf(key.substring(8,9)).intValue()-1; 		
        ((Vraag)Questions.elementAt(nq)).vraag = makeCDATA(value);
       
        // count the number of non-blank questions
        if (((Vraag)Questions.elementAt(nq)).vraag.length()!=0)
          realnumber++;
      } else
      // Are we reading a radio button?	
      if (key.startsWith("radio")) {
        nq=Integer.valueOf(key.substring(5,6)).intValue()-1;
        na=Integer.valueOf(key.substring(6,7)).intValue()-1;  
	((Vraag)Questions.elementAt(nq)).setCorrect(na,value.equalsIgnoreCase("correct"));

      } else 
      // Are we reading an answer?
      if (key.startsWith("answer")) {
        nq=Integer.valueOf(key.substring(6,7)).intValue()-1;
        na=Integer.valueOf(key.substring(7,8)).intValue()-1;
        ((Vraag)Questions.elementAt(nq)).setAntwoord(na,makeCDATA(value));

      } else 
        // Are we reading an explanation?
        if (key.startsWith("explain")) { 
        nq=Integer.valueOf(key.substring(7,8)).intValue()-1;
        na=Integer.valueOf(key.substring(8,9)).intValue()-1;
        ((Vraag)Questions.elementAt(nq)).setUitleg(na,makeCDATA(value));


      } else 
        // Or is it an unknown keyword?
        if ((key.equalsIgnoreCase("number") 
                  || key.equalsIgnoreCase("show")
                  || key.equalsIgnoreCase("title")
                  || key.equalsIgnoreCase("verbose")
                  || key.equalsIgnoreCase("maxanswers"))==false)
  
        throw new AhaException("Unknown form identifier ("+key+"). Please notify the author that this error occured.");		
    }
  }


  public void genXML() {

    // First, add the DTD to the buffer
    addXMLBuffer("<?xml version=\"1.0\"?>");
    addXMLBuffer("<!DOCTYPE test [");
    addXMLBuffer("");
    addXMLBuffer("<!ELEMENT test (question)+>");
    addXMLBuffer("<!ATTLIST test title CDATA #REQUIRED");
    addXMLBuffer("               concept CDATA #REQUIRED");
    addXMLBuffer("               total CDATA #REQUIRED");
    addXMLBuffer("               ask CDATA #REQUIRED");
    addXMLBuffer("               verbose (true | false) #REQUIRED>");
    addXMLBuffer("");
    addXMLBuffer("<!ELEMENT question (#PCDATA|answer)*>");
    addXMLBuffer("<!ATTLIST question answers CDATA #REQUIRED");
    addXMLBuffer("                   right CDATA #REQUIRED>");
    addXMLBuffer("");
    addXMLBuffer("<!ELEMENT answer (#PCDATA|explain)*>");
    addXMLBuffer("<!ATTLIST answer correct (true | false) #REQUIRED>");
    addXMLBuffer("");
    addXMLBuffer("<!ELEMENT explain (#PCDATA)>");
    addXMLBuffer("]>");
    addXMLBuffer("");
  
    Indent="";
    // Now, add the question data to the buffer
    addXMLBuffer("<test title=\""+title+"\"");
    addXMLBuffer("      concept=\"nothing\"");
    addXMLBuffer("      total=\""+String.valueOf(realnumber)+"\"");
    addXMLBuffer("      ask=\""+show+"\"");
    addXMLBuffer("      verbose=\""+verbose.equalsIgnoreCase("true")+"\">"); 
    doIndent();
    

    // Now, add the questions and the answers
    for (int qcount=0;qcount<Integer.valueOf(number).intValue();qcount++) {
 
      // Check if the Question field is empty
      if (((Vraag)Questions.elementAt(qcount)).vraag.length()!=0) {
        // let's check the number of answers
 
        int na=0; // number of answers
        int nc=0; // number of correct answers 
        for (int i=0;i<maxanswers;i++) {
          // Check if answers exists
          if (((Vraag)Questions.elementAt(qcount)).getAntwoord(i).length()!=0) {
            na++;
            // if so, check if it is correct
            if (((Vraag)Questions.elementAt(qcount)).getCorrect(i))
              nc++;
          }
        } 
 
        addXMLBuffer(indent()+"<question answers=\""+na+"\" right=\""+nc+"\">");
        doIndent();
        addXMLBuffer(indent()+((Vraag)Questions.elementAt(qcount)).vraag);
        // Add all the answers to the file
        for (int i=0;i<na;i++) {
          // Check if answer field isn't empty
          if (((Vraag)Questions.elementAt(qcount)).getAntwoord(i).length()!=0) {
            addXMLBuffer(indent()+"<answer correct=\""+((Vraag)Questions.elementAt(qcount)).getCorrect(i)+"\">");           
            doIndent();
            addXMLBuffer(indent()+((Vraag)Questions.elementAt(qcount)).getAntwoord(i));
         
 
            // Check if explain field isn't empty
            if (((Vraag)Questions.elementAt(qcount)).getUitleg(i).length()!=0) {
              addXMLBuffer(indent()+"<explain>");
              doIndent(); 
              addXMLBuffer(indent()+((Vraag)Questions.elementAt(qcount)).getUitleg(i));
              unIndent();
              addXMLBuffer(indent()+"</explain>");
            }
            unIndent();
            addXMLBuffer(indent()+"</answer>");
          }          
        }
        unIndent();  
        addXMLBuffer(indent()+"</question>");
      }
    }
    unIndent();
    addXMLBuffer(indent()+"</test>");  
  } 


  public void addXMLBuffer(String t) {

  XMLBuf.append(t+'\n');
  } 

  /* Pre: filename is a valid filename, ending with ".XML" 
     Post: The test is saved in XML format
  */
  public void write(String filename) 
  throws AhaException {

    try {
      BufferedWriter XMLFile=new BufferedWriter(new FileWriter(filename));
    
      XMLFile.write(XMLBuf.toString());
      XMLFile.close();
    } catch (IOException e) {
      throw new AhaException(e.getMessage());
    }
  }

  public String indent() {
  
  return Indent;
  }
  
  public void doIndent() {
   
  Indent=Indent+"  ";
  }
   
  public void unIndent() {
 
  Indent=Indent.substring(0,Indent.length()-2);
  }

	/**
	 * Set the name field. That name is used for storing the testfile.
	 * ".xml" will end the name.
	 */
	public void setFileName(String filename) {
		this.name = filename;
		if (!name.toLowerCase().endsWith(".xml")) {
			this.name = this.name+".xml";
		}			
	}

	public String getFileName() {
		return this.name;
	}


	/**
	 * makeCDATA returns an XML-CDATA section over the given String.
	 * This is needed because the author may use HTML tags in his questions/answers/explanations.
	 */
	public String makeCDATA(String s) {
		return "<![CDATA[" + s + "]]>";
	}
}
