package AHA;
import org.w3c.dom.*;
import Expression.*;
import Filter.*;

public class Link {
	private AHAclass AHA;
	private Node linkNode;
	private String name;
	private String url;
	private String type;

	final String GOOD = "Good";
	final String BAD = "Bad";
	final String NEUTRAL = "Neutral";

	public Link(AHAclass AHA, Node linkNode) {
		this.AHA = AHA;
		this.linkNode = linkNode;
	}

	public void setAHA(AHAclass AHA) {this.AHA = AHA;}
	public void setNode(Node node) {this.linkNode = node;}

	public String toString() {
		try {
			return FilterLink(linkNode);
		} catch (AhaException ahae) {
			System.out.println("Link error: "+ahae.toString());
			return "LINKERROR";
		}
	}

	/**
	 * Used for adjusting the color of the anchor
	 * @param h name of page
	 */
	public String hasVisited(String h) {
		String href = h;
		if (h.indexOf(".xml")>-1) href = h.substring(0,h.indexOf(".xml"));
		UserModel UserModel = AHA.getUserModel();
		if (UserModel.hasNotVisit(href)) return GOOD;
		if (UserModel.hasBadVisit(href)) return GOOD;
		if (UserModel.hasGoodVisit(href)) return NEUTRAL;
		return "ERROR";
	}

	/** 
	 * Used for adjusting the color of the anchor
	 * @param href name of page
	 */
	public String isGood(String href) throws AhaException {
		ConceptsList cl = AHA.getConceptsList();
		String h = href.substring(0,href.indexOf(".xml"));
		if (cl.containsKey(h)) {
			Concept c = (Concept)cl.get(h);
			Expr expr = c.getRequirements();
			try {
				if (expr.eval(AHA.getUserModel().getKnowledge())) {
					return hasVisited(href);
				} else return BAD;
			} catch (ExprException ee) {
				throw new AhaException(ee.toString());
			}
		}
		else {
			return hasVisited(href);
		}
	}
		
	/**
	 * given an anchorNode change the 'class' attribute according
	 * to the user's knowledge
	 * @param anchorNode the anchor object
	 */
	public String FilterLink(Node anchorNode) throws AhaException{
		
		Element anchor = (Element)anchorNode;
	
		String href = anchor.getAttribute("href");

		String name = anchor.getAttribute("name");
		String aclass = anchor.getAttribute("class");
		String target = anchor.getAttribute("target");
	//	if (anchor.hasAttribute("class")) aclass = anchor.getAttribute("class");
		String other = anchor.getAttribute("other");
		if (aclass == null || aclass.equals("")) {
			if (href.indexOf(":")==-1 && !href.startsWith("test___") && !href.startsWith("eval___")) {
				anchor.setAttribute("class",hasVisited(href));
			} else if (href.startsWith("test___")) {
				anchor.setAttribute("class",GOOD);
			}
		} else {	// class != null
			if (aclass.equalsIgnoreCase("conditional")) {
				if (href.indexOf(":") == -1) {
					anchor.setAttribute("class",isGood(href));
				} else {
					// print warning
					System.out.println("External link with CLASS??");
				}

			} else if (aclass.equalsIgnoreCase("unconditional")) {
				if (href.indexOf(":") == -1) {
					anchor.setAttribute("class",hasVisited(href));
				} else {
					// print warning
					System.out.println("External Link with CLASS?");
				}
			} else {
				throw new AhaException("Unknown anchor class. Please notify the author");
			}
		}
		int l = href.indexOf("DoTest");
		if (l > -1) {
			// this is a link to a test
			UserModel UserModel = AHA.getUserModel();
			String Id = UserModel.getId();
			String Course = AHA.getCourse();
			String begin = href.substring(0, l + 7);
			String eind = href.substring(l+7);
			href = begin+Course+"/"+Id+"/"+eind;
		}
		XmlFilter xf = new XmlFilter(AHA, anchor);
		StringBuffer value = xf.Filter();
		//StringBuffer value = new XmlFilter(AHA).printChildNodes(anchor, true);
		String an = new String("<a ");
		if (!href.equals("")) an += "href=\""+href+"\" ";
		if (!name.equals("")) an += "name=\""+name+"\" ";
		if (anchor.getAttribute("class") != null) 
			an += "class=\""+anchor.getAttribute("class")+"\" ";
		//if (!aclass.equals("")) an += "class=\""+anchor.getAttribute("class")+"\" ";
		if (!target.equals("")) an += "target=\""+target+"\" ";
		if (!other.equals("")) an += "other=\""+other+"\"";
		an += ">"+value+"</a>";
		return an;
	}
	public StringBuffer printChildNodes(Node node, boolean t) {
			return printChildNodes(node, t);
	}

}	
		
