
/*
 * @(#)Messages.java	1.0	98/07/31
 *
 * Copyright (c) 1998 Eindhoven University of Technology. All Rights Rserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import javax.servlet.*;
import javax.servlet.http.*;

	/**
	 * Messages contains common routines for creating headers, footers,
	 * and error messages.
	 */

public class Messages {
	private boolean debug = true;
	private AHAclass AHA;
	private String historyline; // the header line with info on the read pages.


	UserModel UserModel;
	String HtDocs;
	String Course;
	String Logs;
	String Title;
	String Background;
	int Language;
	String Icons;
	String Host;
	int Port;
	String Applets;
	PrintWriter out;
	String Id;


	/**
	 * The default constructor does nothing.
	 * It is not allowed but at this stage nothing is set up to throw exceptions yet.
	 */
	public Messages() {
	}

	/**
	 * The standard constructor only initializes the AHA context.
	 * All other fields must be set later.
	 */
	public Messages(AHAclass AHA) {
		this.AHA = AHA;
		UserModel = AHA.getUserModel();
		HtDocs = AHA.getHtDocs();
		Course = AHA.getCourse();
		Logs = AHA.getLogs();
		Title = AHA.getTitle();
		Background = AHA.getBackground();
		Language = AHA.getLanguage();
		Icons = AHA.getIcons();
		Host = AHA.getHost();
		Port = AHA.getPort();
		Applets = AHA.getApplets();
		out = AHA.getWriter();
		Id = UserModel.getId();
	}

	/**
	 * This method generates a minimal HTML header with a title for an
	 * error message.
	 */
	public void printSimpleHeader() {
	  // Generate required part the HTML header in case something
	  // went wrong, so rely on as little as possible.
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<html>");
		out.println("<head>");
		out.println("<META HTTP-EQUIV=\"Cache-Control\" Content=\"no-cache\">");
		out.println("<META HTTP-EQUIV=\"Expires\" CONTENT=\"-1\">");
		out.println("<title>An Error Occurred.</title>");
		out.println("</head>");
		out.println("<body bgcolor=\"#FFFFFF\">");
	}

	/**
	 * This method dumps a HTML-close (/html) tag.
	 */
	public void printSimpleFooter() {
		out.println("</html>");
	}

	/**
	 * This method generates the required header for every HTML page.
	 * It does not show visible output in the BODY of the document.
	 * It needs the user model to initialize the style sheet with link colors.
	 * The Course, HtDocs, Icons and Background are needed to set the background
	 * image. The Title is needed in the header.
	 * This method is only called by AhaServlet, because it must be done
	 * for all servlets.
	 */
	public void initHeader() {
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<html>");
		out.println("<head>");
		out.println("<META HTTP-EQUIV=\"Cache-Control\" Content=\"no-cache\">");
		out.println("<META HTTP-EQUIV=\"Expires\" CONTENT=\"-1\">");
		out.println("<title>" + Title + "</title>");
		out.println("<style type=\"text/css\">");
		out.println("A.Good { text-decoration: none; color: " + UserModel.printGoodLinkColor() + " }");
		out.println("A.Bad { text-decoration: none; color: " + UserModel.printBadLinkColor() + " }");
		out.println("A.Neutral { text-decoration: none; color: " + UserModel.printNeutralLinkColor() + " }");
		out.println("A.Unconditional { text-decoration: none; color: " + UserModel.printNeutralLinkColor() + " }");
		out.println("</style>");
		out.println("</head>");
	}

	/**
	 * This method generates the body tag in case we do not wish to
	 * print a header.
	 */
	public StringBuffer printBodyTag() {
		StringBuffer sb = new StringBuffer();
		sb.append("<body background=\"/" + Course + "/" + Icons + "/" +
			Background + "\" link=" +
			UserModel.printExternalLinkColor() + " alink=" +
			UserModel.printActiveLinkColor() + " vlink=" +
			UserModel.printExternalVisitedColor() + ">"
		);
		return sb;
	}

	/**
	 * This method generates a /body tag, for the case where no visible
	 * footer is desired.
	 */
	public StringBuffer printEndBodyTag() {
		return new StringBuffer("</body>");
	}

	/**
	 * This method generates an HTTP relocation header.
	 * It may be used by the CGI version but not by the Servlet version.
	 */
	public void relocation(String Url) {
		// Generate a relocation header, so the browser will load the
		// correct first document.
		out.println("Location: " + Url);
		out.println();
	}

	/**
	 * This method prints out an error message (after a minimal header).
	 */
	public void Error(String message) {
		printSimpleHeader();
		out.println("<H2>Error Occurred</H2>");
		out.println(message);
	}

	/**
	 * This method prints out a fatal error message, after a minimal header.
	 */
	public void Fatal(String message) {
		printSimpleHeader();
		out.println("<H2>Fatal Error Occurred</H2>");
		out.println(message);
		printSimpleFooter();
	}

	public static void dumpFatal(PrintWriter out, String message) {
	  // Generate required part the HTML header in case something
	  // went wrong, so rely on as little as possible.
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		out.println("<html>");
		out.println("<head>");
		out.println("<title>An Error Occurred.</title>");
		out.println("</head>");
		out.println("<body bgcolor=\"#FFFFFF\">");		
		out.println("<H2>Fatal Error Occurred</H2>");
		out.println(message);
		out.println("</html>");
	}
}
