/**
 * @(#)PageRequestHandler.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;
import Expression.*;
import java.util.Hashtable;
import java.util.Enumeration;
import Filter.*;
import XmlUtils.*;


/**
 * The class PageRequestHandler takes care of checking/generating
 * knowledge and calling the XmlFilter for a page.
 */

public class PageRequestHandler{
	PrintWriter out;
	private int full = 100;		// 100 percent
	private int part = 35;		// what percentage of knowledege is generated in case of partial generation
	private final int rest = full-part;	// rest
	private AHAclass AHA;
	private String FullDocName;
	private String DocName;
	private String name;
	private ConceptsList ConceptsList;

	/**
	 * If the requested pagename is not valid, an AhaException will be thrown
	 */
	public PageRequestHandler(AHAclass AHA, String FullDocName, String DocName) throws AhaException{
		this.AHA = AHA;
		out = AHA.getWriter();
		this.ConceptsList = AHA.getConceptsList();

		this.FullDocName = FullDocName;
		this.DocName = DocName;
		int p = FullDocName.lastIndexOf(Utils.SEP);
		
		int q = FullDocName.lastIndexOf(".xml");
		if (p>=q) {
			String error = "Due to a page-naming conventions the requested page is not valid.";
			if (FullDocName.indexOf(".xml") == -1) 
				error += " The file extension should be \".xml\". Please notify the author of the course.";
			throw new AhaException(error);
		}
		// the requested pagename is OK
		name = FullDocName.substring(p+1,q);
	}

	/**
	 * Do the job.
	 */
	public void run() throws AhaException{

		// Update the user's knowledge
		// Note: this happens before the user views the page.
		
		CheckAndUpdateKnowledge();

		try {
			// now filter the xml document
			XmlFilter xf = new XmlFilter(AHA, FullDocName, DocName);
			out.println(xf.Filter());
		} catch (Exception e) {
			System.out.println("Error filtering file: "+FullDocName);
			System.out.println(e.toString());
			Messages Messages = AHA.getMessages();
			// CHANGE
			Messages.Fatal("Error reading file: "+FullDocName);
			
		}

		// debug info?????
		Hashtable h = AHA.getUserModel().getVisitedList();
		Enumeration e = h.keys();
		while (e.hasMoreElements()) {
			String key = (String)e.nextElement();
			if (!FulfillRequirements(key)) {
			}
		}
	}

	private void CheckAndUpdateKnowledge2() throws AhaException{
		// this is a test case for the updating process
		// Generate_part . Generate_rest must work well for the whole tree
		Generate g = new Generate(AHA,name,part,false);
		g = new Generate(AHA,name,rest,true);
	}	

	/**
	 * This method checks and changes knowledge for pages/concepts
	 */
	public void CheckAndUpdateKnowledge() throws AhaException{
		boolean b = FulfillRequirements(this.name);
		UserModel UserModel = AHA.getUserModel();

		String clause = "";
		String requirement = "true";
		if (ConceptsList.containsKey(name)) {
			Concept c = (Concept)ConceptsList.get(name);
			clause = c.getGenerates();
			requirement = c.getRequirements().toString();
		}

		if (b) {

			if (UserModel.hasNotVisit(name)) {
				//	FULL

				Hashtable vl = UserModel.getVisitedList();
				
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(full).toString());
				} else {
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);
				Generate g = new Generate(AHA,name,full,true);
			} else if (UserModel.hasBadVisit(name)) {
				//	REST
				Hashtable vl = UserModel.getVisitedList();
				// i don't add the rest, but just put 100% instead
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(full).toString());
				} else {
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);

				Generate g = new Generate(AHA,name,rest,true);
				// set good visit
				// generate_rest
			} else if (UserModel.hasGoodVisit(name)) {
			        // repeat visit
				Hashtable vl = UserModel.getVisitedList();

				Generate g = new Generate(AHA,name,full,true);

			}				
		} else {
			if (UserModel.hasNotVisit(name)) {
				//	PART
				Hashtable vl = UserModel.getVisitedList();
				if (!isSwitch(name, clause)) {
					vl.put(name, new Integer(part).toString());
				} else {
					vl.put(name, new Integer(0).toString());
				}
				UserModel.setVisitedList(vl);
				Generate g = new Generate(AHA,name,part,false);

			} else if (UserModel.hasBadVisit(name)) {
				//	NOTHING
			} else if (UserModel.hasGoodVisit(name)) {
				//	? remove knowledge?
				// what to generate now?
			}
		}

		// now update the pagecounters (pages (not,bad,good)read, total pages) in UserModel
		UserModel.recount();
	}

	public boolean isSwitch(String name, String clause) {
	// returns true if the clause for name contains name:0
		if (clause == null) return false;
		else return clause.indexOf(name+":-100")>-1;
	} 

	public boolean FulfillRequirements(String name) throws AhaException {
	// does the user fulfill the requirements for page name?
		boolean b=false;
		if (!ConceptsList.containsKey(name)) { b = true;
		//	System.out.println("No requirements");
		}
		else {
			Concept c = (Concept)ConceptsList.get(name);
			Expr expr = c.getRequirements();
			//System.out.print(name+ " "+expr.toString()+" ");
			try {
				b = expr.eval(AHA.getUserModel().getKnowledge());
			} catch (ExprException ee) {
				throw new AhaException(ee.toString());
			}
		}
		return b;
	}
}
	
