/**
 * @(#)Registration.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import java.util.Hashtable;
import java.util.Vector;
import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;
import javax.servlet.http.*;
import javax.servlet.*;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * This servlet reads the parameters that the user filled in in 
 * a registration form. 
 */

public class Registration extends Entree {

	Hashtable UserModelNames;


	/**
	 * The init method reads the configured parameters which tell the servlet
	 * where to find documents, log files, background image and document title.
	 *
	 * @exception ServletException        Not thrown purposely by this init
	 * method, but can be generated by the init method of the superclass.
	 */
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		UserModelNames = new Hashtable(100, (float)0.1);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) {
		service(request, response);
	}

	protected void service(HttpServletRequest request, HttpServletResponse response) {
		// First set the contenttype. Necessary in case of
		// an errormessage that has to be displayed.
		// In the early days when the Jigsaw server was being
		// used, this was not necessary.
		response.setContentType("text/html");

		try { 
			// set the PrintWriter and preprocess the form parameters
			super.prepare(request, response);
			
			boolean filledin = (loginid!=null && passwd!=null && HtDocs!=null && Course!=null
                               && Logs!=null && Title!=null && Background!=null && Language!=null);
			if (!filledin) {
				throw new AhaException("Initial parameters are missing.");
			}
			AHA = new AHAclass(HtDocs, Logs, Course, Title, Background, Language);
			AhaProperties AhaProperties = new AhaProperties(AHA);
			AHA.setAhaProperties(AhaProperties);
			Registration(request, response, new Hashtable());
			super.addAHAInformation(request, response);
			super.addSessionInformation(request, response);
			redirect(request, response);
		} catch (Exception e) {
			System.out.println(e.toString());
			log("Registration Error: "+e.toString());
			Messages.dumpFatal(out,this.getName()+" Error: "+e.toString()+"<p>Please notify the author about this error if necessary.");
		}
	}

	/**
	 * This method does some checks and tries to build a usermodel.
	 */
	public void Registration(HttpServletRequest request, HttpServletResponse response, Hashtable UserModelNames) throws AhaException, IOException{
		String FormName; String FormEmail; String FormId;
		String FormPasswd; String FormUniv; String FormDept;

		// no user id and pathname, thus registration form
		if ((FormName = request.getParameter("name")) == null)
			throw new AhaException("Registration form does not contain a 'name' field. This indicates an error in the registration form. Please notify the author that this error occurred.");
		if ((FormEmail = request.getParameter("email")) == null)
			throw new AhaException("Registration form does not contain a 'email' field. This indicates an error in the registration form. Please notify the author that this error occurred.");
		if ((FormId = request.getParameter("id")) == null)
			throw new AhaException("Registration form does not contain an 'id' field. This indicates an error in the registration form. Please notify the author that this error occurred.");
		if ((FormPasswd = request.getParameter("passwd")) == null)
			throw new AhaException("Registration form does not contain a 'passwd' field. This indicates an error in the registration form. Please notify the author that this error occurred.");
		// The following two fields are used in typical AHA courses,
		// but may not be required for some applications.
		if ((FormUniv = request.getParameter("university")) == null)
			FormUniv = "unknown university";
		if ((FormDept = request.getParameter("department")) == null)
			FormDept = "unknown department";

		// The following is used to remove possible whitespace surrounding
		// the values entered in the input fields.
		FormName = FormName.trim();
		FormEmail = FormEmail.trim();
		FormId = FormId.trim();
		FormPasswd = FormPasswd.trim();
		FormUniv = FormUniv.trim();
		FormDept = FormDept.trim();
			if (FormName.length() == 0)
				throw new AhaException("No name was entered. You must provide your name to get access to this hyperdocument.");
			if (FormEmail.length() == 0)
				throw new AhaException("No email address was entered. You must provide your email address to get access to this hyperdocument.");
			if (FormUniv.indexOf("select") != -1)
				throw new AhaException("No university was selected. You must select which university you are from to get access to this hyperdocument.");
			if (FormDept.indexOf("select") != -1)
				throw new AhaException("No department was selected. You must select which department you are from to get access to this hyperdocument.");
			if (FormId.length() == 0)
				throw new AhaException("No identification was entered. You must provide an identification to get access to this hyperdocument.");
			if (FormPasswd.length() == 0)
				throw new AhaException("No password was entered. You must provide a password for authentication procedures.");


		File CourseLogDir = new File(new FileName(Logs, Course).toString());
		if (!CourseLogDir.exists()) {
			try {
			  // Directory for log files does not exist, so we try to create it.
				File LogDir = new File(Logs);
				if (!LogDir.exists()) {
					LogDir.mkdir();
				}
				CourseLogDir.mkdir();
			} catch (Exception e) {
				System.out.println("Could not make Course directory: "+CourseLogDir.toString());
				throw new AhaException("Could not make Course log directory");
			}
		}

		String userlogdir = new FileName(Logs, Course, FormId).toString();
		File UserLogDir = new File(userlogdir);
		if (!UserLogDir.exists()) {
			try {
				UserLogDir.mkdir();
			} catch (Exception e) {
				System.out.println("Could not make personal log directory: "+UserLogDir.toString());
				throw new AhaException("Could not make personal log directory for "+loginid+".");
			}
		}
		if (!UserLogDir.exists())
			throw new AhaException("Log directory (" + userlogdir + ") does not exist and cannot be created. This indicates an error in the configuration of this AHA document. Please notify the author that this error occurred.");
		String LogName = userlogdir;
		// We make the LogName a unique object that can be used for locking.
		if (!UserModelNames.contains(LogName))
			UserModelNames.put(LogName,LogName);
		LogName = (String)UserModelNames.get(LogName);

		// Now we read the logfile and construct the User Model.
		// Note: the Documentlist is needed before this,
		// because the logfile indicates which documents have been read.
		try {
			UserModel = new UserModel(AHA, LogName);
		} catch (AhaException e) {
			System.out.println("Usermodel "+LogName+" doesnt exist yet");
			UserModel = null;
		  // when an exception occurs this indicates the user does not yet
		  // exist (or it means something else is wrong with the User Model)
		  // so we ignore this error.
		  ;
		}

		if (UserModel != null) {
			throw new AhaException("A usermodel with Student ID `"+loginid+"' already exists. "
				+ "Please check your ID, or, in case you forgot your password, send an email "
				+ "to the author of this course.");

		} else {

			if (UserModel == null) {
				UserModel = new UserModel(AHA);
			}
			if (UserModel == null)
				throw new AhaException("Creation of User Model failed. Please notify the author that this error occurred.");
			AHA.setUserModel(UserModel);
			if (FormEmail.indexOf('@') == -1)
				throw new AhaException("The e-mail address must be complete, including your domain. (The form of a complete e-mail address is 'user@domain'.)");
			if (FormPasswd.equalsIgnoreCase(FormName) ||
				FormPasswd.equalsIgnoreCase(FormEmail) ||
				FormPasswd.equalsIgnoreCase(FormEmail.substring(0, FormEmail.indexOf('@'))) ||
				FormPasswd.equals(FormId))
					throw new AhaException("The chosen password is too easy to guess. Please try again with a more difficult password.");
			UserModel.setName(FormName);
			UserModel.setEmail(FormEmail);
			UserModel.setId(FormId);
			UserModel.setPasswd(FormPasswd);
			UserModel.setInstitute(FormUniv);
			UserModel.setDepartment(FormDept);
			UserModel.setKnowledge(new Hashtable(500,(float)0.1));
			UserModel.setVisitedList(buildNewPageList());
			UserModel.setLogFileLines(new Vector());
			UserModel.saveUserModel();
		}
	}

	/** 
	 * This returns a hashtable with all concepts initialized
	 * on zero.
	 */
	private Hashtable buildNewPageList() throws AhaException{
		Hashtable pages = new Hashtable();
		String filename="";
		try {
			filename = AHA.getDocumentsListFile();
		} catch (Exception e) {
			throw new AhaException("Error");
		}
		if (filename == null) 
			throw new AhaException("Error in properties file: no DocumentsList setting given.");
		if (filename.endsWith(new FileName("",null).toString()))
			throw new AhaException("Error in properties file: no DocumentsList setting given.");
		File documentslist = new File(filename);
		if (!documentslist.canRead()) 
			throw new AhaException("Cannot read DocumentList file: "+filename);
		try {
			BufferedReader fin = new BufferedReader(new FileReader(filename));
			while (fin.ready()) {
				String page = fin.readLine();
				pages.put(page ,new Integer(0).toString());
			}
			fin.close();
		} catch (IOException ioe) {
			System.out.println("Error reading pagelist file: "+filename);
			throw new AhaException("Error initializing UserModel: could not read the list of pages in "+Course+" from file \""+filename+"\". <p>Please notify the author of the course.");
		}
		return pages;
	}		
}
