package Filter;

import XmlUtils.*;
import Expression.*;
import org.w3c.dom.*;
import AHA.*;

public class XmlFilter {
	public AHAclass AHA;
	Node node;
	String nodename;

	// callingdocname is een variabele waarin de naam staat
	// van de pagina waarin het Stop-applet staat.
	// Die variabele is een parameter voor het Stop-applet.
	// Dat applet stuurt de waarde naar het Stop-servlet
	// die het weer gebruikt bij het wegschrijven van log-informatie.
	// callingdocname is static, want dat voorkomt een hoop rompslomp.
	public static String callingdocname;

	private final String ifstring = "if";
	private final String headerstring = "header";
	private final String footerstring = "footer";
	private final String astring = "a";
	private final String blockstring = "block";
	final String FILEATTR = "file";
	final String PRINTATTR = "print";
	final String SERVLETSTRING = "servlet";
	final String VARIABLESTRING = "variable";
	final String STOPAPPLETSTRING = "stopapplet";

	public XmlFilter(AHAclass AHA, String filename, String callingdocname) throws AhaException {
		this(AHA, filename);
		this.callingdocname = callingdocname;
	}

	public XmlFilter(AHAclass AHA, String filename) throws AhaException {
		try {
			XmlReader xr = new XmlReader(filename);
			this.node = xr.getRoot();
			this.nodename = node.getNodeName();
			this.AHA = AHA;
		} catch (Exception e) {
			throw new AhaException(e.toString());
		}
	}
		
	public XmlFilter(AHAclass AHA, Node n) {
		node = n;
		nodename = node.getNodeName();
		this.AHA = AHA;
		this.callingdocname = callingdocname;
	}

	public String printName() {
		return "Nodename="+nodename;
	}

	public StringBuffer Filter() throws AhaException {
		StringBuffer sb = new StringBuffer();
		NodeList nodelist = node.getChildNodes();
		for (int i=0; i<nodelist.getLength(); i++) {
			Node child = nodelist.item(i);
			XmlFilter f;
			if (isIfTag(child)) {
				f = new IfFilter(AHA, child);
			} else if (isHeaderTag(child)) {
				f = new HeaderFilter(AHA, child);
			} else if (isFooterTag(child)) {
				f = new FooterFilter(AHA, child);
			} else if (isATag(child)) {
				f = new AFilter(AHA, child);
			} else if (isBlockTag(child)) {
				f = new BlockFilter(AHA, child);
			} else if (isServletTag(child)) {
				f = new ServletFilter(AHA, child);
			} else if (isVariableTag(child)) {
				f = new VariableFilter(AHA, child);
			} else if (isStopAppletTag(child)) {

				f = new StopAppletFilter(AHA, child);
			} else {
				f = new CDATAFilter(AHA, child);
			}
			StringBuffer sbtemp = f.Filter();
			sb = sb.append(sbtemp);
		}
		return sb;
	}

	public boolean isServletTag(Node node) {
		return node.getNodeName().equals(SERVLETSTRING);
	}
	public boolean isVariableTag(Node node) {
		return node.getNodeName().equals(VARIABLESTRING);
	}
	public boolean isStopAppletTag(Node node) {
		return node.getNodeName().equals(STOPAPPLETSTRING);
	}
	public boolean isIfTag(Node node) {
		return node.getNodeName().equals(ifstring);
	}
	public boolean isHeaderTag(Node node) {
		return node.getNodeName().equals(headerstring);
	}
	public boolean isFooterTag(Node node) {
		return node.getNodeName().equals(footerstring);
	}
	public boolean isATag(Node node) {
		return node.getNodeName().equals(astring);
	}
	public boolean isBlockTag(Node node) {
		return node.getNodeName().equals(blockstring);
	}

	public String getName() {
		return getClass().getName();
	}
	public String getCallingDocName() {
		return callingdocname;
	}
}	
	
