package XmlUtils;

import java.io.*;
import com.sun.xml.tree.*;
import com.sun.xml.parser.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *
 * A standard reader for reading XML data files
 *
 */

public class XmlReader {
	private XmlDocument doc;
	private String xmlfile;
	private boolean validate = true;
	private String error = null;

	public void print(Object s) {System.out.print(s);}
	public void println(Object s) {System.out.println(s);}

	/**
	 * default constructor; not valid
	 */

	public XmlReader() {
		println("Error: wrong class acces");
	}

	/**
	 * The XmlReader method actually reads the file with name name.
	 * @param v if v then a validating parser is used
	 * @exception Exception if the XML code is non valid, or the file does not exist then an exception is thrown
	 */
	public XmlReader(String name, boolean v) throws Exception {
		this.validate = v;
		this.xmlfile = name;
		System.out.println("XmlReader: "+xmlfile);
		InputSource input;
		try {
			input = Resolver.createInputSource(new File(xmlfile));
			this.doc = XmlDocument.createXmlDocument(input,validate);
		} catch (SAXParseException err) {
			System.out.println("Error parsing document: "+xmlfile+ "\n"+", line " + err.getLineNumber ()+ ", uri " + err.getSystemId ());
			System.out.println("   " + err.getMessage ());
			error = new String("Error parsing document: "+xmlfile+ "\n"+", line " + err.getLineNumber ()+ ", uri " + err.getSystemId ());
			error = error + new String("   " + err.getMessage ());
			throw new Exception(error);
		} catch (Exception e) { 
			System.out.println(e);
			error = new String("File not found: "+xmlfile);
			throw new Exception(error);
		}
	}

	/** 
	 * This constructor calls a validating parser for name
	 * @exception Exception passes the exceptions thrown
	 */
	public XmlReader(String name) throws Exception {
			this(name, true);
	}

	/**
	 * get the root Element of the XML data file
	 */
	public Element getRoot() {
		return (Element)doc.getDocumentElement();
	}

	/**
	 * getFirst returns the first element with name name
	 * @param nodename The name for the element
	 */
	public String getFirst(String nodename) {
		Node n = getRoot().getElementsByTagName(nodename).item(0);
		return n.toString();
		//return s = doc.getElementsByTagName(nodename).item(0);
	}

	/**
	 * returns the error
	 */
	public String getError() { return error;}

	/** 
	 * This method is used for testing only
	 */
	public static void main(String args[]) {
		try {
			XmlReader xr = new XmlReader(args[0]);
			Element el = xr.getRoot();
			System.out.println(el.toString());
		} catch (Exception e) {
			System.out.println(e.toString());
		}
	}	

}		